@@ -1,16 +1,15 @@
 //! Contains functionality to load an ArrayData from the C Data Interface
 use std::sync::Arc;
 
+use polars_buffer::{Buffer, SharedStorage};
 use polars_error::{PolarsResult, polars_bail};
 
 use super::ArrowArray;
 use crate::array::*;
 use crate::bitmap::Bitmap;
 use crate::bitmap::utils::bytes_for;
-use crate::buffer::Buffer;
 use crate::datatypes::{ArrowDataType, PhysicalType};
 use crate::ffi::schema::get_child;
-use crate::storage::SharedStorage;
 use crate::types::{NativeType, PrimitiveType, months_days_ns};
 use crate::{ffi, match_integer_type, with_match_primitive_type_full};
 
@@ -260,7 +259,8 @@ unsafe fn create_buffer_known_len<T: NativeType>(
         return Ok(Buffer::new());
     }
     let ptr: *mut T = get_buffer_ptr(array, dtype, index)?;
-    let storage = SharedStorage::from_internal_arrow_array(ptr, len, owner);
+    let slice = core::slice::from_raw_parts(ptr, len);
+    let storage = SharedStorage::from_slice_with_owner(slice, owner);
     Ok(Buffer::from_storage(storage))
 }
 
@@ -275,9 +275,9 @@ unsafe fn create_buffer<T: NativeType>(
     owner: InternalArrowArray,
     index: usize,
 ) -> PolarsResult<Buffer<T>> {
-    let len = buffer_len(array, dtype, index)?;
+    let buf_len = buffer_len(array, dtype, index)?;
 
-    if len == 0 {
+    if buf_len == 0 {
         // Zero-length arrays might have invalid pointers for zero-length slices in Rust,
         // so this is more than just an optimization.
         return Ok(Buffer::new());
@@ -285,18 +285,23 @@ unsafe fn create_buffer<T: NativeType>(
 
     let offset = buffer_offset(array, dtype, index);
     let ptr: *mut T = get_buffer_ptr(array, dtype, index)?;
+    let len = buf_len - offset;
 
-    // We have to check alignment.
-    // This is the zero-copy path.
-    if ptr.align_offset(align_of::<T>()) == 0 {
-        let storage = SharedStorage::from_internal_arrow_array(ptr, len, owner);
-        Ok(Buffer::from_storage(storage).sliced(offset, len - offset))
-    }
-    // This is the path where alignment isn't correct.
-    // We copy the data to a new vec
-    else {
-        let buf = std::slice::from_raw_parts(ptr, len - offset).to_vec();
-        Ok(Buffer::from(buf))
+    // We have to check alignment, for zero-copy to be valid.
+    if ptr.is_aligned() {
+        let slice = core::slice::from_raw_parts(ptr.add(offset), len);
+        let storage = SharedStorage::from_slice_with_owner(slice, owner);
+        Ok(Buffer::from_storage(storage))
+    } else {
+        // Byte-wise copy for misaligned buffers.
+        let mut v = Vec::with_capacity(len);
+        core::ptr::copy_nonoverlapping(
+            ptr.add(offset).cast::<u8>(),
+            v.spare_capacity_mut().as_mut_ptr().cast::<u8>(),
+            len * size_of::<T>(),
+        );
+        v.set_len(len);
+        Ok(Buffer::from(v))
     }
 }
 
@@ -323,10 +328,10 @@ unsafe fn create_bitmap(
     let ptr = get_buffer_ptr(array, dtype, index)?;
 
     // Pointer of u8 has alignment 1, so we don't have to check alignment.
-
     let offset: usize = array.offset.try_into().expect("offset to fit in `usize`");
     let bytes_len = bytes_for(offset + len);
-    let storage = SharedStorage::from_internal_arrow_array(ptr, bytes_len, owner);
+    let slice = core::slice::from_raw_parts(ptr, bytes_len);
+    let storage = SharedStorage::from_slice_with_owner(slice, owner);
 
     let null_count = if is_validity {
         Some(array.null_count())
