@@ -14,7 +14,6 @@ pub struct QuantileWindow<'a, T: NativeType> {
 }
 
 impl<
-    'a,
     T: NativeType
         + Float
         + std::iter::Sum
@@ -26,35 +25,38 @@ impl<
         + Zero
         + SealedRolling
         + Sub<Output = T>,
-> RollingAggWindowNoNulls<'a, T> for QuantileWindow<'a, T>
+> RollingAggWindowNoNulls<T> for QuantileWindow<'_, T>
 {
-    fn new(
+    type This<'a> = QuantileWindow<'a, T>;
+
+    fn new<'a>(
         slice: &'a [T],
         start: usize,
         end: usize,
         params: Option<RollingFnParams>,
         window_size: Option<usize>,
-    ) -> Self {
+    ) -> Self::This<'a> {
         let params = params.unwrap();
         let RollingFnParams::Quantile(params) = params else {
             unreachable!("expected Quantile params");
         };
 
-        Self {
+        QuantileWindow {
             sorted: SortedBuf::new(slice, start, end, window_size),
             prob: params.prob,
             method: params.method,
         }
     }
 
-    unsafe fn update(&mut self, start: usize, end: usize) -> Option<T> {
+    unsafe fn update(&mut self, start: usize, end: usize) {
         self.sorted.update(start, end);
-        let length = self.sorted.len();
+    }
 
+    fn get_agg(&self, _idx: usize) -> Option<T> {
+        let length = self.sorted.len();
         if length == 0 {
             return None;
-        };
-
+        }
         let idx = match self.method {
             Linear => {
                 // Maybe add a fast path for median case? They could branch depending on odd/even.
@@ -101,6 +103,10 @@ impl<
 
         Some(self.sorted.get(idx))
     }
+
+    fn slice_len(&self) -> usize {
+        self.sorted.slice_len()
+    }
 }
 
 pub fn rolling_quantile<T>(
