@@ -1,3 +1,4 @@
+use core::panic;
 use std::marker::PhantomData;
 
 use polars_utils::IdxSize;
@@ -9,27 +10,29 @@ use super::*;
 pub struct RankWindow<'a, T, Out, P> {
     slice: &'a [T],
     validity: &'a Bitmap,
-    last_start: usize,
-    last_end: usize,
+    start: usize,
+    end: usize,
     ost: OrderStatisticTree<&'a T>,
     policy: P,
     _out: PhantomData<Out>,
 }
 
-impl<'a, T, Out, P> RollingAggWindowNulls<'a, T, Out> for RankWindow<'a, T, Out, P>
+impl<T, Out, P> RollingAggWindowNulls<T, Out> for RankWindow<'_, T, Out, P>
 where
     T: NativeType,
     Out: NativeType,
     P: RankPolicy<T, Out>,
 {
-    fn new(
+    type This<'a> = RankWindow<'a, T, Out, P>;
+
+    fn new<'a>(
         slice: &'a [T],
         validity: &'a Bitmap,
         start: usize,
         end: usize,
         params: Option<RollingFnParams>,
         window_size: Option<usize>,
-    ) -> Self {
+    ) -> Self::This<'a> {
         assert!(start <= slice.len() && end <= slice.len() && start <= end);
 
         let cmp = |a: &&T, b: &&T| T::tot_cmp(*a, *b);
@@ -37,37 +40,37 @@ where
             Some(ws) => OrderStatisticTree::with_capacity(ws, cmp),
             None => OrderStatisticTree::new(cmp),
         };
-        let mut slf = Self {
+        let mut this = RankWindow {
             slice,
             validity,
-            last_start: 0,
-            last_end: 0,
+            start: 0,
+            end: 0,
             ost,
             policy: P::new(&params.unwrap()),
             _out: PhantomData,
         };
         // SAFETY: We bounds checked `start` and `end`.
         unsafe {
-            slf.update(start, end);
+            this.update(start, end);
         }
-        slf
+        this
     }
 
-    unsafe fn update(&mut self, new_start: usize, new_end: usize) -> Option<Out> {
-        debug_assert!(self.last_start <= self.last_end);
-        debug_assert!(self.last_end <= self.slice.len());
+    unsafe fn update(&mut self, new_start: usize, new_end: usize) {
+        debug_assert!(self.start <= self.end);
+        debug_assert!(self.end <= self.slice.len());
         debug_assert!(new_start <= new_end);
         debug_assert!(new_end <= self.slice.len());
-        debug_assert!(self.last_start <= new_start);
-        debug_assert!(self.last_end <= new_end);
+        debug_assert!(self.start <= new_start);
+        debug_assert!(self.end <= new_end);
 
-        for i in self.last_end..new_end {
+        for i in self.end..new_end {
             if !self.validity.get(i).unwrap() {
                 continue;
             }
             self.ost.insert(unsafe { self.slice.get_unchecked(i) });
         }
-        for i in self.last_start..new_start {
+        for i in self.start..new_start {
             if !self.validity.get(i).unwrap() {
                 continue;
             }
@@ -75,22 +78,31 @@ where
                 .remove(&unsafe { self.slice.get_unchecked(i) })
                 .expect("previously added value is missing");
         }
-        self.last_start = new_start;
-        self.last_end = new_end;
-        let cur = unsafe { self.slice.get_unchecked(self.last_end - 1) };
-        self.policy.rank(&self.ost, cur)
+        self.start = new_start;
+        self.end = new_end;
+    }
+
+    fn get_agg(&self, idx: usize) -> Option<Out> {
+        if !(self.start..self.end).contains(&idx) {
+            panic!("index out of bounds");
+        }
+        self.policy.rank(&self.ost, &self.slice[idx])
     }
 
     fn is_valid(&self, _min_periods: usize) -> bool {
-        self.validity.get(self.last_end - 1).unwrap()
+        self.validity.get(self.end - 1).unwrap()
+    }
+
+    fn slice_len(&self) -> usize {
+        self.slice.len()
     }
 }
 
-type RankWindowAvg<'a, T> = RankWindow<'a, T, f64, RankPolicyAverage>;
-type RankWindowMin<'a, T> = RankWindow<'a, T, IdxSize, RankPolicyMin>;
-type RankWindowMax<'a, T> = RankWindow<'a, T, IdxSize, RankPolicyMax>;
-type RankWindowDense<'a, T> = RankWindow<'a, T, IdxSize, RankPolicyDense>;
-type RankWindowRandom<'a, T> = RankWindow<'a, T, IdxSize, RankPolicyRandom>;
+pub type RankWindowAvg<'a, T> = RankWindow<'a, T, f64, RankPolicyAverage>;
+pub type RankWindowMin<'a, T> = RankWindow<'a, T, IdxSize, RankPolicyMin>;
+pub type RankWindowMax<'a, T> = RankWindow<'a, T, IdxSize, RankPolicyMax>;
+pub type RankWindowDense<'a, T> = RankWindow<'a, T, IdxSize, RankPolicyDense>;
+pub type RankWindowRandom<'a, T> = RankWindow<'a, T, IdxSize, RankPolicyRandom>;
 
 pub fn rolling_rank<T>(
     arr: &PrimitiveArray<T>,
