@@ -81,6 +81,14 @@ pub trait ChunkAnyValue {
     fn get_any_value(&self, index: usize) -> PolarsResult<AnyValue<'_>>;
 }
 
+pub trait ChunkAnyValueBypassValidity {
+    /// Get a single value bypassing the validity map. Beware this is slow.
+    ///
+    /// # Safety
+    /// Does not do any bounds checking.
+    unsafe fn get_any_value_bypass_validity(&self, index: usize) -> AnyValue<'_>;
+}
+
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
@@ -293,6 +301,14 @@ pub trait ChunkQuantile<T> {
     fn quantile(&self, _quantile: f64, _method: QuantileMethod) -> PolarsResult<Option<T>> {
         Ok(None)
     }
+    /// Aggregate a given set of quantiles of the ChunkedArray.
+    /// Returns `None` if the array is empty or only contains null values.
+    fn quantiles(&self, quantiles: &[f64], _method: QuantileMethod) -> PolarsResult<Vec<Option<T>>>
+    where
+        T: Clone,
+    {
+        Ok(vec![None; quantiles.len()])
+    }
 }
 
 /// Variance and standard deviation aggregation.
