@@ -488,7 +488,7 @@ pub fn _df_rows_to_hashes_threaded_vertical(
             .map(|df| {
                 let hb = build_hasher.clone();
                 let mut hashes = vec![];
-                columns_to_hashes(df.columns(), Some(hb), &mut hashes)?;
+                columns_to_hashes(df, Some(hb), &mut hashes)?;
                 Ok(UInt64Chunked::from_vec(PlSmallStr::EMPTY, hashes))
             })
             .collect::<PolarsResult<Vec<_>>>()
@@ -497,13 +497,23 @@ pub fn _df_rows_to_hashes_threaded_vertical(
 }
 
 pub fn columns_to_hashes(
-    keys: &[Column],
+    keys: &DataFrame,
     build_hasher: Option<PlSeedableRandomStateQuality>,
     hashes: &mut Vec<u64>,
 ) -> PolarsResult<PlSeedableRandomStateQuality> {
     let build_hasher = build_hasher.unwrap_or_default();
 
-    let mut iter = keys.iter();
+    if keys.width() == 0 {
+        let null_h = get_null_hash_value(&build_hasher);
+
+        for _ in 0..keys.height() {
+            hashes.push(null_h);
+        }
+
+        return Ok(build_hasher);
+    }
+
+    let mut iter = keys.columns().iter();
     let first = iter.next().expect("at least one key");
     first.vec_hash(build_hasher.clone(), hashes)?;
 
