@@ -1090,6 +1090,12 @@ impl Series {
     }
 }
 
+impl Default for Series {
+    fn default() -> Self {
+        NullChunked::new(PlSmallStr::EMPTY, 0).into_series()
+    }
+}
+
 impl Deref for Series {
     type Target = dyn SeriesTrait;
 
@@ -1104,12 +1110,6 @@ impl<'a> AsRef<dyn SeriesTrait + 'a> for Series {
     }
 }
 
-impl Default for Series {
-    fn default() -> Self {
-        Int64Chunked::default().into_series()
-    }
-}
-
 impl<T: PolarsPhysicalType> AsRef<ChunkedArray<T>> for dyn SeriesTrait + '_ {
     fn as_ref(&self) -> &ChunkedArray<T> {
         // @NOTE: SeriesTrait `as_any` returns a std::any::Any for the underlying ChunkedArray /
