@@ -19,12 +19,11 @@ use object_store::azure::MicrosoftAzureBuilder;
 use object_store::gcp::GoogleCloudStorageBuilder;
 #[cfg(feature = "gcp")]
 pub use object_store::gcp::GoogleConfigKey;
-#[cfg(any(feature = "aws", feature = "gcp", feature = "azure"))]
-use object_store::{BackoffConfig, RetryConfig};
 use polars_error::*;
 #[cfg(feature = "aws")]
 use polars_utils::cache::LruCache;
 use polars_utils::pl_path::{CloudScheme, PlRefPath};
+use polars_utils::total_ord::TotalOrdWrap;
 #[cfg(feature = "http")]
 use reqwest::header::HeaderMap;
 #[cfg(feature = "serde")]
@@ -79,10 +78,11 @@ pub(crate) enum CloudConfig {
 #[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 /// Options to connect to various cloud providers.
 pub struct CloudOptions {
-    pub max_retries: usize,
     #[cfg(feature = "file_cache")]
     pub file_cache_ttl: u64,
     pub(crate) config: Option<CloudConfig>,
+    #[cfg_attr(feature = "serde", serde(default))]
+    pub retry_config: CloudRetryConfig,
     #[cfg(feature = "cloud")]
     /// Note: In most cases you will want to access this via [`CloudOptions::initialized_credential_provider`]
     /// rather than directly.
@@ -98,10 +98,10 @@ impl Default for CloudOptions {
 impl CloudOptions {
     pub fn default_static_ref() -> &'static Self {
         static DEFAULT: LazyLock<CloudOptions> = LazyLock::new(|| CloudOptions {
-            max_retries: 2,
             #[cfg(feature = "file_cache")]
             file_cache_ttl: get_env_file_cache_ttl(),
             config: None,
+            retry_config: CloudRetryConfig::default(),
             #[cfg(feature = "cloud")]
             credential_provider: None,
         });
@@ -110,6 +110,78 @@ impl CloudOptions {
     }
 }
 
+#[derive(Clone, Copy, Default, Debug, PartialEq, Hash, Eq)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
+pub struct CloudRetryConfig {
+    pub max_retries: Option<usize>,
+    pub retry_timeout: Option<std::time::Duration>,
+    pub retry_init_backoff: Option<std::time::Duration>,
+    pub retry_max_backoff: Option<std::time::Duration>,
+    pub retry_base_multiplier: Option<TotalOrdWrap<f64>>,
+}
+
+#[cfg(any(feature = "aws", feature = "gcp", feature = "azure"))]
+impl From<CloudRetryConfig> for object_store::RetryConfig {
+    fn from(value: CloudRetryConfig) -> Self {
+        use std::time::Duration;
+
+        use polars_core::config::verbose;
+
+        let out = object_store::RetryConfig {
+            backoff: object_store::BackoffConfig {
+                init_backoff: value
+                    .retry_init_backoff
+                    .unwrap_or_else(|| DEFAULTS.backoff.init_backoff),
+                max_backoff: value
+                    .retry_max_backoff
+                    .unwrap_or_else(|| DEFAULTS.backoff.max_backoff),
+                base: value
+                    .retry_base_multiplier
+                    .map_or_else(|| DEFAULTS.backoff.base, |x| x.0),
+            },
+            max_retries: value.max_retries.unwrap_or_else(|| DEFAULTS.max_retries),
+            retry_timeout: value
+                .retry_timeout
+                .unwrap_or_else(|| DEFAULTS.retry_timeout),
+        };
+
+        if verbose() {
+            eprintln!("object-store retry config: {:?}", &out)
+        }
+
+        return out;
+
+        static DEFAULTS: LazyLock<object_store::RetryConfig> =
+            LazyLock::new(|| object_store::RetryConfig {
+                backoff: object_store::BackoffConfig {
+                    init_backoff: Duration::from_millis(parse_env_var(
+                        100,
+                        "POLARS_CLOUD_RETRY_INIT_BACKOFF_MS",
+                    )),
+                    max_backoff: Duration::from_millis(parse_env_var(
+                        15 * 1000,
+                        "POLARS_CLOUD_RETRY_MAX_BACKOFF_MS",
+                    )),
+                    base: parse_env_var(2., "POLARS_CLOUD_RETRY_BASE_MULTIPLIER"),
+                },
+                max_retries: parse_env_var(2, "POLARS_CLOUD_MAX_RETRIES"),
+                retry_timeout: Duration::from_millis(parse_env_var(
+                    10 * 1000,
+                    "POLARS_CLOUD_RETRY_TIMEOUT_MS",
+                )),
+            });
+
+        fn parse_env_var<T: FromStr>(default: T, name: &'static str) -> T {
+            std::env::var(name).map_or(default, |x| {
+                x.parse::<T>()
+                    .ok()
+                    .unwrap_or_else(|| panic!("invalid value for {name}: {x}"))
+            })
+        }
+    }
+}
+
 #[cfg(feature = "http")]
 pub(crate) fn try_build_http_header_map_from_items_slice<S: AsRef<str>>(
     headers: &[(S, S)],
@@ -182,15 +254,6 @@ impl CloudType {
     }
 }
 
-#[cfg(any(feature = "aws", feature = "gcp", feature = "azure"))]
-fn get_retry_config(max_retries: usize) -> RetryConfig {
-    RetryConfig {
-        backoff: BackoffConfig::default(),
-        max_retries,
-        retry_timeout: std::time::Duration::from_secs(10),
-    }
-}
-
 pub static USER_AGENT: &str = concat!("polars", "/", env!("CARGO_PKG_VERSION"),);
 
 #[cfg(any(feature = "aws", feature = "gcp", feature = "azure", feature = "http"))]
@@ -253,9 +316,8 @@ fn read_config(
 }
 
 impl CloudOptions {
-    /// Set the maximum number of retries.
-    pub fn with_max_retries(mut self, max_retries: usize) -> Self {
-        self.max_retries = max_retries;
+    pub fn with_retry_config(mut self, retry_config: CloudRetryConfig) -> Self {
+        self.retry_config = retry_config;
         self
     }
 
@@ -399,7 +461,7 @@ impl CloudOptions {
             };
         };
 
-        let builder = builder.with_retry(get_retry_config(self.max_retries));
+        let builder = builder.with_retry(self.retry_config.into());
 
         let opt_credential_provider = match opt_credential_provider {
             #[cfg(feature = "python")]
@@ -473,7 +535,7 @@ impl CloudOptions {
 
         let builder = builder
             .with_url(url.to_string())
-            .with_retry(get_retry_config(self.max_retries));
+            .with_retry(self.retry_config.into());
 
         let builder =
             if let Some(v) = self.initialized_credential_provider(clear_cached_credentials)? {
@@ -535,7 +597,7 @@ impl CloudOptions {
 
         let builder = builder
             .with_url(url.to_string())
-            .with_retry(get_retry_config(self.max_retries));
+            .with_retry(self.retry_config.into());
 
         let builder = if let Some(v) = credential_provider {
             builder.with_credentials(v.into_gcp_provider())
