@@ -66,3 +66,28 @@ pub fn get_upload_chunk_size() -> usize {
         v
     });
 }
+
+pub fn get_upload_concurrency() -> usize {
+    use std::sync::LazyLock;
+
+    return *UPLOAD_CONCURRENCY;
+
+    static UPLOAD_CONCURRENCY: LazyLock<usize> = LazyLock::new(|| {
+        // Max number of parts concurrently uploaded per Writer.
+        // @NOTE. The object_store::BufWriter uses 8 as default.
+        let v = std::env::var("POLARS_UPLOAD_CONCURRENCY")
+            .map(|x| {
+                x.parse::<usize>()
+                    .ok()
+                    .filter(|x| *x > 0)
+                    .unwrap_or_else(|| panic!("invalid value for POLARS_UPLOAD_CONCURRENCY: {x}"))
+            })
+            .unwrap_or(8);
+
+        if polars_core::config::verbose() {
+            eprintln!("async upload_concurrency: {v}")
+        }
+
+        v
+    });
+}
