@@ -29,14 +29,6 @@ pub enum AggExpr {
         input: Arc<Expr>,
         propagate_nans: bool,
     },
-    MinBy {
-        input: Arc<Expr>,
-        by: Arc<Expr>,
-    },
-    MaxBy {
-        input: Arc<Expr>,
-        by: Arc<Expr>,
-    },
     Median(Arc<Expr>),
     NUnique(Arc<Expr>),
     First(Arc<Expr>),
@@ -69,8 +61,8 @@ impl AsRef<Expr> for AggExpr {
     fn as_ref(&self) -> &Expr {
         use AggExpr::*;
         match self {
-            Min { input, .. } | MinBy { input, .. } => input,
-            Max { input, .. } | MaxBy { input, .. } => input,
+            Min { input, .. } => input,
+            Max { input, .. } => input,
             Median(e) => e,
             NUnique(e) => e,
             First(e) => e,
@@ -183,6 +175,16 @@ pub enum Expr {
     Len,
     #[cfg(feature = "dtype-struct")]
     Field(Arc<[PlSmallStr]>),
+    AnonymousAgg {
+        /// function arguments
+        input: Vec<Expr>,
+        /// function to apply
+        function: OpaqueStreamingAgg,
+
+        /// used for formatting
+        #[cfg_attr(any(feature = "serde", feature = "dsl-schema"), serde(skip))]
+        fmt_str: Box<PlSmallStr>,
+    },
     AnonymousFunction {
         /// function arguments
         input: Vec<Expr>,
@@ -409,6 +411,14 @@ impl Hash for Expr {
                 function.hash(state);
                 expr.hash(state);
             },
+            Expr::AnonymousAgg {
+                input,
+                function: _,
+                fmt_str,
+            } => {
+                input.hash(state);
+                fmt_str.hash(state);
+            },
             Expr::AnonymousFunction {
                 input,
                 function: _,
@@ -717,8 +727,11 @@ pub enum Operator {
     Plus,
     Minus,
     Multiply,
-    Divide,
+    /// Rust division semantics, this is what Rust interface `/` fispatches to
+    RustDivide,
+    /// Python division semantics, converting to floats. This is what python `/` operator dispatches to
     TrueDivide,
+    /// Floor division semantics, this is what python `//` dispatches to
     FloorDivide,
     Modulus,
     And,
@@ -743,9 +756,9 @@ impl Display for Operator {
             Plus => "+",
             Minus => "-",
             Multiply => "*",
-            Divide => "//",
+            RustDivide => "rust_div",
             TrueDivide => "/",
-            FloorDivide => "floor_div",
+            FloorDivide => "//",
             Modulus => "%",
             And | LogicalAnd => "&",
             Or | LogicalOr => "|",
@@ -792,7 +805,7 @@ impl Operator {
             Operator::EqValidity => Operator::EqValidity,
             Operator::NotEqValidity => Operator::NotEqValidity,
             // Operator::Divide requires modifying the right operand: left / right == 1/right * left
-            Operator::Divide => unimplemented!(),
+            Operator::RustDivide => unimplemented!(),
             Operator::Multiply => Operator::Multiply,
             Operator::And => Operator::And,
             Operator::Plus => Operator::Plus,
