@@ -107,6 +107,11 @@ pub enum IRStringFunction {
     // DataType can only be Date/Datetime/Time
     Strptime(DataType, StrptimeOptions),
     Split(bool),
+    #[cfg(feature = "regex")]
+    SplitRegex {
+        inclusive: bool,
+        strict: bool,
+    },
     #[cfg(feature = "dtype-decimal")]
     ToDecimal {
         scale: usize,
@@ -190,7 +195,9 @@ impl IRStringFunction {
                 },
                 _ => mapper.with_dtype(dtype.clone()),
             },
-            Split(_) => mapper.with_dtype(DataType::List(Box::new(DataType::String))),
+            Split(_) => mapper.with_dtype(DataType::List(DataType::String.into())),
+            #[cfg(feature = "regex")]
+            SplitRegex { .. } => mapper.with_dtype(DataType::List(DataType::String.into())),
             #[cfg(feature = "nightly")]
             Titlecase => mapper.with_same_dtype(),
             #[cfg(feature = "dtype-decimal")]
@@ -297,6 +304,8 @@ impl IRStringFunction {
             S::SplitExact { .. } => FunctionOptions::elementwise(),
             #[cfg(feature = "dtype-struct")]
             S::SplitN(_) => FunctionOptions::elementwise(),
+            #[cfg(feature = "regex")]
+            S::SplitRegex { .. } => FunctionOptions::elementwise(),
             #[cfg(feature = "find_many")]
             S::ContainsAny { .. } => FunctionOptions::elementwise(),
             #[cfg(feature = "find_many")]
@@ -385,6 +394,14 @@ impl Display for IRStringFunction {
                     "split"
                 }
             },
+            #[cfg(feature = "regex")]
+            SplitRegex { inclusive, .. } => {
+                if *inclusive {
+                    "split_regex_inclusive"
+                } else {
+                    "split_regex"
+                }
+            },
             #[cfg(feature = "nightly")]
             Titlecase => "to_titlecase",
             #[cfg(feature = "dtype-decimal")]
