@@ -87,13 +87,13 @@ impl Hash for AExpr {
                 truthy: _,
                 falsy: _,
             } => {},
-            AExpr::AnonymousStreamingAgg {
+            AExpr::AnonymousAgg {
                 input: _,
                 fmt_str,
-                function: _,
+                function,
             } => {
+                function.hash(state);
                 fmt_str.hash(state);
-                // Invariant, fmt_str is unique. Only used in cloud.
             },
             AExpr::Eval {
                 expr: _,
