@@ -1,29 +1,32 @@
 use std::io::BufReader;
-use std::sync::LazyLock;
+use std::sync::{LazyLock, RwLock};
 
-use arrow::buffer::Buffer;
 use either::Either;
+use polars_buffer::Buffer;
 use polars_io::RowIndex;
 use polars_io::csv::read::streaming::read_until_start_and_infer_schema;
-#[cfg(feature = "cloud")]
-use polars_io::pl_async::get_runtime;
 use polars_io::prelude::*;
 use polars_io::utils::compression::CompressedReader;
 
 use super::*;
 
-pub(super) fn dsl_to_ir(
+pub(super) async fn dsl_to_ir(
     sources: ScanSources,
     mut unified_scan_args_box: Box<UnifiedScanArgs>,
     scan_type: Box<FileScanDsl>,
     cached_ir: Arc<Mutex<Option<IR>>>,
-    ctxt: &mut DslConversionContext,
-) -> PolarsResult<IR> {
+    cache_file_info: SourcesToFileInfo,
+    verbose: bool,
+) -> PolarsResult<()> {
     // Note that the first metadata can still end up being `None` later if the files were
     // filtered from predicate pushdown.
-    let mut cached_ir = cached_ir.lock().unwrap();
+    // Check and drop the lock in its own scope
+    let is_not_cached = {
+        let cached_ir_guard = cached_ir.lock().unwrap();
+        cached_ir_guard.is_none()
+    };
 
-    if cached_ir.is_none() {
+    if is_not_cached {
         let unified_scan_args = unified_scan_args_box.as_mut();
 
         if let Some(hive_schema) = unified_scan_args.hive_options.schema.as_deref() {
@@ -46,14 +49,20 @@ pub(super) fn dsl_to_ir(
         let sources = match &*scan_type {
             #[cfg(feature = "parquet")]
             FileScanDsl::Parquet { .. } => {
-                sources.expand_paths_with_hive_update(unified_scan_args)?
+                sources
+                    .expand_paths_with_hive_update(unified_scan_args)
+                    .await?
             },
             #[cfg(feature = "ipc")]
-            FileScanDsl::Ipc { .. } => sources.expand_paths_with_hive_update(unified_scan_args)?,
+            FileScanDsl::Ipc { .. } => {
+                sources
+                    .expand_paths_with_hive_update(unified_scan_args)
+                    .await?
+            },
             #[cfg(feature = "csv")]
-            FileScanDsl::Csv { .. } => sources.expand_paths(unified_scan_args)?,
+            FileScanDsl::Csv { .. } => sources.expand_paths(unified_scan_args).await?,
             #[cfg(feature = "json")]
-            FileScanDsl::NDJson { .. } => sources.expand_paths(unified_scan_args)?,
+            FileScanDsl::NDJson { .. } => sources.expand_paths(unified_scan_args).await?,
             #[cfg(feature = "python")]
             FileScanDsl::PythonDataset { .. } => {
                 // There are a lot of places that short-circuit if the paths is empty,
@@ -61,19 +70,23 @@ pub(super) fn dsl_to_ir(
                 ScanSources::Paths(Buffer::from_iter([PlRefPath::new("PL_PY_DSET")]))
             },
             #[cfg(feature = "scan_lines")]
-            FileScanDsl::Lines { .. } => sources.expand_paths(unified_scan_args)?,
+            FileScanDsl::Lines { .. } => sources.expand_paths(unified_scan_args).await?,
             FileScanDsl::Anonymous { .. } => sources.clone(),
         };
 
         // For cloud we must deduplicate files. Serialization/deserialization leads to Arc's losing there
         // sharing.
-        let (mut file_info, scan_type_ir) = ctxt.cache_file_info.get_or_insert(
-            &scan_type,
-            &sources,
-            sources_before_expansion,
-            unified_scan_args,
-            ctxt.verbose,
-        )?;
+        let (mut file_info, scan_type_ir) = {
+            cache_file_info
+                .get_or_insert(
+                    &scan_type,
+                    &sources,
+                    sources_before_expansion,
+                    unified_scan_args,
+                    verbose,
+                )
+                .await?
+        };
 
         if unified_scan_args.hive_options.enabled.is_none() {
             // We expect this to be `Some(_)` after this point. If it hasn't been auto-enabled
@@ -168,10 +181,11 @@ pub(super) fn dsl_to_ir(
             }
         };
 
+        let mut cached_ir = cached_ir.lock().unwrap();
         cached_ir.replace(ir);
     }
 
-    Ok(cached_ir.clone().unwrap())
+    Ok(())
 }
 
 pub(super) fn insert_row_index_to_schema(
@@ -219,7 +233,7 @@ fn prepare_schemas(
 }
 
 #[cfg(feature = "parquet")]
-pub(super) fn parquet_file_info(
+pub(super) async fn parquet_file_info(
     first_scan_source: ScanSourceRef<'_>,
     row_index: Option<&RowIndex>,
     #[allow(unused)] cloud_options: Option<&polars_io::cloud::CloudOptions>,
@@ -231,17 +245,14 @@ pub(super) fn parquet_file_info(
         if first_scan_source.is_cloud_url() {
             let first_path = first_scan_source.as_path().unwrap();
             feature_gated!("cloud", {
-                get_runtime().block_in_place_on(async {
-                    let mut reader =
-                        ParquetObjectStore::from_uri(first_path.clone(), cloud_options, None)
-                            .await?;
+                let mut reader =
+                    ParquetObjectStore::from_uri(first_path.clone(), cloud_options, None).await?;
 
-                    PolarsResult::Ok((
-                        reader.schema().await?,
-                        reader.num_rows().await?,
-                        reader.get_metadata().await?.clone(),
-                    ))
-                })?
+                (
+                    reader.schema().await?,
+                    reader.num_rows().await?,
+                    reader.get_metadata().await?.clone(),
+                )
             })
         } else {
             let memslice = first_scan_source.to_memslice()?;
@@ -284,7 +295,7 @@ pub fn max_metadata_scan_cached() -> usize {
 
 // TODO! return metadata arced
 #[cfg(feature = "ipc")]
-pub(super) fn ipc_file_info(
+pub(super) async fn ipc_file_info(
     first_scan_source: ScanSourceRef<'_>,
     row_index: Option<&RowIndex>,
     cloud_options: Option<&polars_io::cloud::CloudOptions>,
@@ -295,12 +306,10 @@ pub(super) fn ipc_file_info(
         ScanSourceRef::Path(path) => {
             if path.has_scheme() {
                 feature_gated!("cloud", {
-                    get_runtime().block_on(async {
-                        polars_io::ipc::IpcReaderAsync::from_uri(path.clone(), cloud_options)
-                            .await?
-                            .metadata()
-                            .await
-                    })?
+                    polars_io::ipc::IpcReaderAsync::from_uri(path.clone(), cloud_options)
+                        .await?
+                        .metadata()
+                        .await?
                 })
             } else {
                 arrow::io::ipc::read::read_file_metadata(&mut std::io::BufReader::new(
@@ -329,7 +338,7 @@ pub(super) fn ipc_file_info(
 }
 
 #[cfg(feature = "csv")]
-pub fn csv_file_info(
+pub async fn csv_file_info(
     sources: &ScanSources,
     _first_scan_source: ScanSourceRef<'_>,
     row_index: Option<&RowIndex>,
@@ -353,10 +362,13 @@ pub fn csv_file_info(
     let cache_entries = {
         if run_async {
             feature_gated!("cloud", {
-                Some(polars_io::file_cache::init_entries_from_uri_list(
-                    sources.as_paths().unwrap().iter().cloned(),
-                    cloud_options,
-                )?)
+                Some(
+                    polars_io::file_cache::init_entries_from_uri_list(
+                        sources.as_paths().unwrap().iter().cloned(),
+                        cloud_options,
+                    )
+                    .await?,
+                )
             })
         } else {
             None
@@ -365,7 +377,7 @@ pub fn csv_file_info(
 
     let infer_schema_func = |i| {
         let source = sources.at(i);
-        let mem_slice = source.to_memslice_possibly_async(run_async, cache_entries.as_ref(), i)?;
+        let mem_slice = source.to_buffer_possibly_async(run_async, cache_entries.as_ref(), i)?;
         let mut reader = CompressedReader::try_new(mem_slice)?;
 
         let mut first_row_len = 0;
@@ -393,7 +405,7 @@ pub fn csv_file_info(
                     (_, true) => Ok((schema_a, row_estimate_a)),
                     _ => {
                         schema_a.to_supertype(&schema_b)?;
-                        Ok((schema_a, row_estimate_a + row_estimate_b))
+                        Ok((schema_a, row_estimate_a.saturating_add(row_estimate_b)))
                     },
                 }
             },
@@ -435,7 +447,7 @@ pub fn csv_file_info(
 }
 
 #[cfg(feature = "json")]
-pub fn ndjson_file_info(
+pub async fn ndjson_file_info(
     sources: &ScanSources,
     first_scan_source: ScanSourceRef<'_>,
     row_index: Option<&RowIndex>,
@@ -450,10 +462,13 @@ pub fn ndjson_file_info(
     let cache_entries = {
         if run_async {
             feature_gated!("cloud", {
-                Some(polars_io::file_cache::init_entries_from_uri_list(
-                    sources.as_paths().unwrap().iter().cloned(),
-                    cloud_options,
-                )?)
+                Some(
+                    polars_io::file_cache::init_entries_from_uri_list(
+                        sources.as_paths().unwrap().iter().cloned(),
+                        cloud_options,
+                    )
+                    .await?,
+                )
             })
         } else {
             None
@@ -464,7 +479,7 @@ pub fn ndjson_file_info(
         schema
     } else {
         let mem_slice =
-            first_scan_source.to_memslice_possibly_async(run_async, cache_entries.as_ref(), 0)?;
+            first_scan_source.to_buffer_possibly_async(run_async, cache_entries.as_ref(), 0)?;
         let mut reader = BufReader::new(CompressedReader::try_new(mem_slice)?);
 
         Arc::new(polars_io::ndjson::infer_schema(
@@ -504,14 +519,14 @@ enum CachedSourceKey {
     },
 }
 
-#[derive(Default)]
+#[derive(Default, Clone)]
 pub(super) struct SourcesToFileInfo {
-    inner: PlHashMap<CachedSourceKey, (FileInfo, FileScanIR)>,
+    inner: Arc<RwLock<PlHashMap<CachedSourceKey, (FileInfo, FileScanIR)>>>,
 }
 
 impl SourcesToFileInfo {
-    fn infer_or_parse(
-        &mut self,
+    async fn infer_or_parse(
+        &self,
         scan_type: FileScanDsl,
         sources: &ScanSources,
         sources_before_expansion: &ScanSources,
@@ -550,7 +565,7 @@ impl SourcesToFileInfo {
                         },
                     )
                 } else {
-                    (|| {
+                    {
                         let first_scan_source = require_first_source(
                             "failed to retrieve first file schema (parquet)",
                             "\
@@ -570,24 +585,25 @@ this scan to succeed with an empty DataFrame.",
                             unified_scan_args.row_index.as_ref(),
                             cloud_options,
                             n_sources,
-                        )?;
+                        )
+                        .await?;
 
                         if let Some((total, deleted)) = unified_scan_args.row_count {
                             let size = (total - deleted) as usize;
                             file_info.row_estimation = (Some(size), size);
                         }
 
-                        if self.inner.len() > max_metadata_scan_cached() {
+                        if self.inner.read().unwrap().len() > max_metadata_scan_cached() {
                             _ = metadata.take();
                         }
 
                         PolarsResult::Ok((file_info, FileScanIR::Parquet { options, metadata }))
-                    })()
+                    }
                     .map_err(|e| e.context(failed_here!(parquet scan)))?
                 }
             },
             #[cfg(feature = "ipc")]
-            FileScanDsl::Ipc { options } => (|| {
+            FileScanDsl::Ipc { options } => {
                 let first_scan_source =
                     require_first_source("failed to retrieve first file schema (ipc)", "")?;
 
@@ -602,7 +618,8 @@ this scan to succeed with an empty DataFrame.",
                     first_scan_source,
                     unified_scan_args.row_index.as_ref(),
                     cloud_options,
-                )?;
+                )
+                .await?;
 
                 PolarsResult::Ok((
                     file_info,
@@ -611,11 +628,11 @@ this scan to succeed with an empty DataFrame.",
                         metadata: Some(Arc::new(md)),
                     },
                 ))
-            })()
+            }
             .map_err(|e| e.context(failed_here!(ipc scan)))?,
             #[cfg(feature = "csv")]
             FileScanDsl::Csv { mut options } => {
-                (|| {
+                {
                     // TODO: This is a hack. We conditionally set `allow_missing_columns` to
                     // mimic existing behavior, but this should be taken from a user provided
                     // parameter instead.
@@ -646,15 +663,16 @@ this scan to succeed with an empty DataFrame.",
                             unified_scan_args.row_index.as_ref(),
                             Arc::make_mut(&mut options),
                             cloud_options,
-                        )?
+                        )
+                        .await?
                     };
 
                     PolarsResult::Ok((file_info, FileScanIR::Csv { options }))
-                })()
+                }
                 .map_err(|e| e.context(failed_here!(csv scan)))?
             },
             #[cfg(feature = "json")]
-            FileScanDsl::NDJson { options } => (|| {
+            FileScanDsl::NDJson { options } => {
                 let file_info = if let Some(schema) = options.schema.clone() {
                     FileInfo {
                         schema: schema.clone(),
@@ -678,11 +696,12 @@ this scan to succeed with an empty DataFrame.",
                         unified_scan_args.row_index.as_ref(),
                         &options,
                         cloud_options,
-                    )?
+                    )
+                    .await?
                 };
 
                 PolarsResult::Ok((file_info, FileScanIR::NDJson { options }))
-            })()
+            }
             .map_err(|e| e.context(failed_here!(ndjson scan)))?,
             #[cfg(feature = "python")]
             FileScanDsl::PythonDataset { dataset_object } => (|| {
@@ -733,8 +752,8 @@ this scan to succeed with an empty DataFrame.",
         })
     }
 
-    pub(super) fn get_or_insert(
-        &mut self,
+    pub(super) async fn get_or_insert(
+        &self,
         scan_type: &FileScanDsl,
         sources: &ScanSources,
         sources_before_expansion: &ScanSources,
@@ -746,16 +765,18 @@ this scan to succeed with an empty DataFrame.",
             ScanSources::Paths(paths) if !paths.is_empty() => paths.clone(),
 
             _ => {
-                return self.infer_or_parse(
-                    scan_type.clone(),
-                    sources,
-                    sources_before_expansion,
-                    unified_scan_args,
-                );
+                return self
+                    .infer_or_parse(
+                        scan_type.clone(),
+                        sources,
+                        sources_before_expansion,
+                        unified_scan_args,
+                    )
+                    .await;
             },
         };
 
-        let (k, v): (CachedSourceKey, Option<&(FileInfo, FileScanIR)>) = match scan_type {
+        let (k, v): (CachedSourceKey, Option<(FileInfo, FileScanIR)>) = match scan_type {
             #[cfg(feature = "parquet")]
             FileScanDsl::Parquet { options } => {
                 let key = CachedSourceKey::ParquetIpc {
@@ -763,8 +784,9 @@ this scan to succeed with an empty DataFrame.",
                     schema_overwrite: options.schema.clone(),
                 };
 
-                let v = self.inner.get(&key);
-                (key, v)
+                let guard = self.inner.read().unwrap();
+                let v = guard.get(&key);
+                (key, v.cloned())
             },
             #[cfg(feature = "ipc")]
             FileScanDsl::Ipc { options: _ } => {
@@ -773,8 +795,9 @@ this scan to succeed with an empty DataFrame.",
                     schema_overwrite: None,
                 };
 
-                let v = self.inner.get(&key);
-                (key, v)
+                let guard = self.inner.read().unwrap();
+                let v = guard.get(&key);
+                (key, v.cloned())
             },
             #[cfg(feature = "csv")]
             FileScanDsl::Csv { options } => {
@@ -783,8 +806,9 @@ this scan to succeed with an empty DataFrame.",
                     schema: options.schema.clone(),
                     schema_overwrite: options.schema_overwrite.clone(),
                 };
-                let v = self.inner.get(&key);
-                (key, v)
+                let guard = self.inner.read().unwrap();
+                let v = guard.get(&key);
+                (key, v.cloned())
             },
             #[cfg(feature = "json")]
             FileScanDsl::NDJson { options } => {
@@ -793,16 +817,19 @@ this scan to succeed with an empty DataFrame.",
                     schema: options.schema.clone(),
                     schema_overwrite: options.schema_overwrite.clone(),
                 };
-                let v = self.inner.get(&key);
-                (key, v)
+                let guard = self.inner.read().unwrap();
+                let v = guard.get(&key);
+                (key, v.cloned())
             },
             _ => {
-                return self.infer_or_parse(
-                    scan_type.clone(),
-                    sources,
-                    sources_before_expansion,
-                    unified_scan_args,
-                );
+                return self
+                    .infer_or_parse(
+                        scan_type.clone(),
+                        sources,
+                        sources_before_expansion,
+                        unified_scan_args,
+                    )
+                    .await;
             },
         };
 
@@ -810,15 +837,17 @@ this scan to succeed with an empty DataFrame.",
             if verbose {
                 eprintln!("FILE_INFO CACHE HIT")
             }
-            Ok(out.clone())
+            Ok(out)
         } else {
-            let v = self.infer_or_parse(
-                scan_type.clone(),
-                sources,
-                sources_before_expansion,
-                unified_scan_args,
-            )?;
-            self.inner.insert(k, v.clone());
+            let v = self
+                .infer_or_parse(
+                    scan_type.clone(),
+                    sources,
+                    sources_before_expansion,
+                    unified_scan_args,
+                )
+                .await?;
+            self.inner.write().unwrap().insert(k, v.clone());
             Ok(v)
         }
     }
