@@ -105,25 +105,6 @@ pub fn node_to_expr(node: Node, expr_arena: &Arena<AExpr>) -> Expr {
                 }
                 .into()
             },
-            IRAggExpr::MinBy { input, by } => {
-                let input_exp = node_to_expr(input, expr_arena);
-                let by_exp = node_to_expr(by, expr_arena);
-                AggExpr::MinBy {
-                    input: Arc::new(input_exp),
-                    by: Arc::new(by_exp),
-                }
-                .into()
-            },
-
-            IRAggExpr::MaxBy { input, by } => {
-                let input_exp = node_to_expr(input, expr_arena);
-                let by_exp = node_to_expr(by, expr_arena);
-                AggExpr::MaxBy {
-                    input: Arc::new(input_exp),
-                    by: Arc::new(by_exp),
-                }
-                .into()
-            },
 
             IRAggExpr::Mean(expr) => {
                 let exp = node_to_expr(expr, expr_arena);
@@ -222,6 +203,15 @@ pub fn node_to_expr(node: Node, expr_arena: &Arena<AExpr>) -> Expr {
                 falsy: Arc::new(f),
             }
         },
+        AExpr::AnonymousAgg {
+            input,
+            fmt_str,
+            function,
+        } => Expr::AnonymousAgg {
+            input: expr_irs_to_exprs(input, expr_arena),
+            function,
+            fmt_str,
+        },
         AExpr::AnonymousFunction {
             input,
             function,
@@ -300,7 +290,6 @@ pub fn node_to_expr(node: Node, expr_arena: &Arena<AExpr>) -> Expr {
             length: Arc::new(node_to_expr(length, expr_arena)),
         },
         AExpr::Len => Expr::Len,
-        AExpr::AnonymousStreamingAgg { .. } => unreachable!("should not be hit"),
     }
 }
 
@@ -530,6 +519,8 @@ pub fn ir_function_to_dsl(input: Vec<Expr>, function: IRFunctionExpr) -> Expr {
                     B::Strptime(dtype.into(), strptime_options)
                 },
                 IB::Split(v) => B::Split(v),
+                #[cfg(feature = "regex")]
+                IB::SplitRegex { inclusive, strict } => B::SplitRegex { inclusive, strict },
                 #[cfg(feature = "dtype-decimal")]
                 IB::ToDecimal { scale } => B::ToDecimal { scale },
                 #[cfg(feature = "nightly")]
@@ -950,6 +941,8 @@ pub fn ir_function_to_dsl(input: Vec<Expr>, function: IRFunctionExpr) -> Expr {
             descending,
             nulls_last,
         },
+        IF::MinBy => F::MinBy,
+        IF::MaxBy => F::MaxBy,
         IF::Product => F::Product,
         #[cfg(feature = "rank")]
         IF::Rank { options, seed } => F::Rank { options, seed },
