@@ -61,8 +61,6 @@ impl TreeWalker for Expr {
             Agg(agg_expr) => Agg(match agg_expr {
                 Min { input, propagate_nans } => Min { input: am(input, f)?, propagate_nans },
                 Max { input, propagate_nans } => Max { input: am(input, f)?, propagate_nans },
-                MinBy { input, by } => MinBy { input: am(input, &mut f)?, by: am(by, f)? },
-                MaxBy { input, by } =>  MaxBy { input: am(input, &mut f)?, by: am(by, f)? },
                 Median(x) => Median(am(x, f)?),
                 NUnique(x) => NUnique(am(x, f)?),
                 First(x) => First(am(x, f)?),
@@ -78,6 +76,7 @@ impl TreeWalker for Expr {
                 AggGroups(x) => AggGroups(am(x, f)?),
                 Std(x, ddf) => Std(am(x, f)?, ddf),
                 Var(x, ddf) => Var(am(x, f)?, ddf),
+
             }),
             Ternary { predicate, truthy, falsy } => Ternary { predicate: am(predicate, &mut f)?, truthy: am(truthy, &mut f)?, falsy: am(falsy, f)? },
             Function { input, function } => Function { input: input.into_iter().map(f).collect::<Result<_, _>>()?, function },
@@ -94,6 +93,9 @@ impl TreeWalker for Expr {
             Element => Element,
             Len => Len,
             RenameAlias { function, expr } => RenameAlias { function, expr: am(expr, f)? },
+            AnonymousAgg { input, function, fmt_str } => {
+                AnonymousAgg { input: input.into_iter().map(f).collect::<Result<_, _>>()?, function, fmt_str }
+            },
             AnonymousFunction { input, function, options, fmt_str } => {
                 AnonymousFunction { input: input.into_iter().map(f).collect::<Result<_, _>>()?, function, options, fmt_str }
             },
@@ -249,7 +251,43 @@ impl AExpr {
                     all_same_name
                 }
             },
-            (AnonymousFunction { .. }, AnonymousFunction { .. }) => false,
+            (
+                AnonymousFunction {
+                    function: l1,
+                    options: l2,
+                    fmt_str: l3,
+                    input: _,
+                },
+                AnonymousFunction {
+                    function: r1,
+                    options: r2,
+                    fmt_str: r3,
+                    input: _,
+                },
+            ) => {
+                l2 == r2 && l3 == r3 && {
+                    use LazySerde as L;
+                    match (l1, r1) {
+                        // We only check the pointers, so this works for python
+                        // functions that are on the same address.
+                        (L::Deserialized(l0), L::Deserialized(r0)) => l0 == r0,
+                        (L::Bytes(l0), L::Bytes(r0)) => l0 == r0,
+                        (
+                            L::Named {
+                                name: l_name,
+                                payload: l_payload,
+                                value: l_value,
+                            },
+                            L::Named {
+                                name: r_name,
+                                payload: r_payload,
+                                value: r_value,
+                            },
+                        ) => l_name == r_name && l_payload == r_payload && l_value == r_value,
+                        _ => false,
+                    }
+                }
+            },
             (BinaryExpr { op: l, .. }, BinaryExpr { op: r, .. }) => l == r,
             _ => false,
         }
