@@ -18,7 +18,7 @@ use crate::error::PyPolarsErr;
 use crate::prelude::strings_to_pl_smallstr;
 use crate::py_modules::polars;
 use crate::series::{PySeries, ToPySeries, ToSeries};
-use crate::utils::EnterPolarsExt;
+use crate::utils::{EnterPolarsExt, to_py_err};
 
 #[pymethods]
 impl PyDataFrame {
@@ -31,6 +31,15 @@ impl PyDataFrame {
         Ok(PyDataFrame::new(df))
     }
 
+    #[staticmethod]
+    pub fn empty_with_height(height: u64) -> PyResult<Self> {
+        Ok(PyDataFrame::new(DataFrame::empty_with_height(
+            IdxSize::try_from(height)
+                .map_err(|_| polars_err!(bigidx, ctx = "DataFrame(height = _)", size = height))
+                .map_err(to_py_err)? as usize,
+        )))
+    }
+
     pub fn estimated_size(&self) -> usize {
         self.df.read().estimated_size()
     }
@@ -311,7 +320,8 @@ impl PyDataFrame {
     pub fn slice(&self, py: Python<'_>, offset: i64, length: Option<usize>) -> PyResult<Self> {
         py.enter_polars_df(|| {
             let df = self.df.read();
-            Ok(df.slice(offset, length.unwrap_or_else(|| df.height())))
+            let len = length.unwrap_or(usize::MAX);
+            Ok(df.slice(offset, len))
         })
     }
 
