@@ -31,13 +31,13 @@ pub fn concat_df(dfs: &Bound<'_, PyAny>, py: Python) -> PyResult<PyDataFrame> {
         polars_core::POOL.install(|| {
             rdfs.into_par_iter()
                 .fold(identity, |acc: PolarsResult<DataFrame>, df| {
-                    let mut acc = acc?;
-                    acc.vstack_mut(&df?)?;
+                    let mut acc: DataFrame = acc?;
+                    acc.vstack_mut_owned(df?)?;
                     Ok(acc)
                 })
                 .reduce(identity, |acc, df| {
                     let mut acc = acc?;
-                    acc.vstack_mut(&df?)?;
+                    acc.vstack_mut_owned(df?)?;
                     Ok(acc)
                 })
         })
