@@ -4,7 +4,7 @@ use polars::prelude::sync_on_close::SyncOnCloseType;
 use polars::prelude::{CloudScheme, UnifiedSinkArgs};
 use pyo3::prelude::*;
 
-use crate::functions::parse_cloud_options;
+use crate::io::cloud_options::OptPyCloudOptions;
 use crate::prelude::Wrap;
 
 /// Interface to `class SinkOptions` on the Python side
@@ -24,13 +24,12 @@ impl PySinkOptions<'_> {
         cloud_scheme: Option<CloudScheme>,
     ) -> PyResult<UnifiedSinkArgs> {
         #[derive(FromPyObject)]
-        struct Extract {
+        struct Extract<'a> {
             mkdir: bool,
             maintain_order: bool,
             sync_on_close: Option<Wrap<SyncOnCloseType>>,
-            storage_options: Option<Vec<(String, String)>>,
+            storage_options: OptPyCloudOptions<'a>,
             credential_provider: Option<Py<PyAny>>,
-            retries: usize,
         }
 
         let Extract {
@@ -39,11 +38,10 @@ impl PySinkOptions<'_> {
             sync_on_close,
             storage_options,
             credential_provider,
-            retries,
         } = self.0.extract()?;
 
         let cloud_options =
-            parse_cloud_options(cloud_scheme, storage_options, credential_provider, retries)?;
+            storage_options.extract_opt_cloud_options(cloud_scheme, credential_provider)?;
 
         let sync_on_close = sync_on_close.map_or(SyncOnCloseType::default(), |x| x.0);
 
