@@ -14,13 +14,12 @@ mod graph;
 pub use graph::{GraphNodeKey, LogicalPipe, LogicalPipeKey};
 pub use skeleton::{QueryResult, StreamingQuery};
 mod metrics;
+mod metrics_io;
 pub use metrics::{GraphMetrics, NodeMetrics};
 mod morsel;
 mod nodes;
 mod physical_plan;
-#[cfg(feature = "physical_plan_visualization")]
-pub use physical_plan::visualization as physical_plan_visualization;
-pub use physical_plan::{PhysNode, PhysNodeKey, PhysNodeKind, ZipBehavior};
+pub use physical_plan::{NodeStyle, PhysNode, PhysNodeKey, PhysNodeKind, ZipBehavior};
 mod pipe;
 mod utils;
 
