@@ -6,23 +6,19 @@ use polars_core::prelude::{DataType, PlHashMap, PlHashSet};
 use polars_core::scalar::Scalar;
 use polars_core::schema::Schema;
 use polars_core::{SchemaExtPl, config};
-use polars_error::{PolarsResult, polars_bail};
+use polars_error::PolarsResult;
 use polars_expr::state::ExecutionState;
 use polars_mem_engine::create_physical_plan;
 use polars_plan::constants::get_literal_name;
 use polars_plan::dsl::default_values::DefaultFieldValues;
 use polars_plan::dsl::deletion::DeletionFilesList;
-use polars_plan::dsl::sink2::FileProviderType;
-use polars_plan::dsl::{
-    CallbackSinkType, ExtraColumnsPolicy, FileScanIR, PartitionStrategyIR, PartitionVariantIR,
-    PartitionedSinkOptionsIR, SinkOptions, SinkTypeIR, UnifiedSinkArgs,
-};
+use polars_plan::dsl::{CallbackSinkType, ExtraColumnsPolicy, FileScanIR, SinkTypeIR};
 use polars_plan::plans::expr_ir::{ExprIR, OutputName};
 use polars_plan::plans::{
     AExpr, FunctionIR, IR, IRAggExpr, LiteralValue, are_keys_sorted_any, is_sorted,
     write_ir_non_recursive,
 };
-use polars_plan::prelude::GroupbyOptions;
+use polars_plan::prelude::*;
 use polars_utils::arena::{Arena, Node};
 use polars_utils::itertools::Itertools;
 use polars_utils::pl_str::PlSmallStr;
@@ -40,6 +36,7 @@ use crate::nodes::io_sources::multi_scan;
 use crate::nodes::io_sources::multi_scan::components::forbid_extra_columns::ForbidExtraColumns;
 use crate::nodes::io_sources::multi_scan::components::projection::builder::ProjectionBuilder;
 use crate::nodes::io_sources::multi_scan::reader_interface::builder::FileReaderBuilder;
+use crate::nodes::joins::merge_join;
 use crate::physical_plan::ZipBehavior;
 use crate::physical_plan::lower_expr::{ExprCache, build_select_stream, lower_exprs};
 use crate::physical_plan::lower_group_by::build_group_by_stream;
@@ -160,10 +157,15 @@ pub fn lower_ir(
     expr_cache: &mut ExprCache,
     cache_nodes: &mut PlHashMap<UniqueId, PhysStream>,
     ctx: StreamingLowerIRContext,
+    mut disable_morsel_split: Option<bool>,
 ) -> PolarsResult<PhysStream> {
     // Helper macro to simplify recursive calls.
     macro_rules! lower_ir {
-        ($input:expr) => {
+        ($input:expr) => {{
+            // Disable for remaining execution graph if it wasn't explicitly set
+            // by the current IR.
+            disable_morsel_split.get_or_insert(false);
+
             lower_ir(
                 $input,
                 ir_arena,
@@ -173,14 +175,21 @@ pub fn lower_ir(
                 expr_cache,
                 cache_nodes,
                 ctx,
+                disable_morsel_split,
             )
-        };
+        }};
+    }
+
+    // Require the code below to explicitly set this to `true`
+    if disable_morsel_split == Some(true) {
+        disable_morsel_split.take();
     }
 
     let ir_node = ir_arena.get(node);
     let output_schema = IR::schema_with_cache(node, ir_arena, schema_cache);
     let node_kind = match ir_node {
         IR::SimpleProjection { input, columns } => {
+            disable_morsel_split.get_or_insert(true);
             let columns = columns.iter_names_cloned().collect::<Vec<_>>();
             let phys_input = lower_ir!(*input)?;
             PhysNodeKind::SimpleProjection {
@@ -191,6 +200,14 @@ pub fn lower_ir(
 
         IR::Select { input, expr, .. } => {
             let selectors = expr.clone();
+
+            if selectors
+                .iter()
+                .all(|e| matches!(expr_arena.get(e.node()), AExpr::Len | AExpr::Column(_)))
+            {
+                disable_morsel_split.get_or_insert(true);
+            }
+
             let phys_input = lower_ir!(*input)?;
             return build_select_stream(
                 phys_input, &selectors, expr_arena, phys_sm, expr_cache, ctx,
@@ -225,7 +242,10 @@ pub fn lower_ir(
             ..
         } => {
             let schema = schema.clone(); // This is initially the schema of df, but can change with the projection.
-            let mut node_kind = PhysNodeKind::InMemorySource { df: df.clone() };
+            let mut node_kind = PhysNodeKind::InMemorySource {
+                df: df.clone(),
+                disable_morsel_split: disable_morsel_split.unwrap_or(true),
+            };
 
             // Do we need to apply a projection?
             if let Some(projection_schema) = projection {
@@ -248,6 +268,7 @@ pub fn lower_ir(
 
         IR::Sink { input, payload } => match payload {
             SinkTypeIR::Memory => {
+                disable_morsel_split.get_or_insert(true);
                 let phys_input = lower_ir!(*input)?;
                 PhysNodeKind::InMemorySink { input: phys_input }
             },
@@ -274,147 +295,15 @@ pub fn lower_ir(
                 PhysNodeKind::FileSink { input, options }
             },
 
-            SinkTypeIR::Partitioned(options)
-                if !matches!(options.file_path_provider, FileProviderType::Legacy(_))
-                    && options.finish_callback.is_none()
-                    && match options.file_format {
-                        #[cfg(feature = "parquet")]
-                        polars_plan::dsl::FileWriteFormat::Parquet(_) => true,
-                        #[cfg(feature = "ipc")]
-                        polars_plan::dsl::FileWriteFormat::Ipc(_) => true,
-                        #[cfg(feature = "csv")]
-                        polars_plan::dsl::FileWriteFormat::Csv(_) => true,
-                        #[cfg(feature = "json")]
-                        polars_plan::dsl::FileWriteFormat::NDJson(_) => true,
-                        #[cfg(not(any(
-                            feature = "parquet",
-                            feature = "ipc",
-                            feature = "csv",
-                            feature = "json"
-                        )))]
-                        _ => panic!("no enum variants on FileType (hint: missing feature flags?)"),
-                    } =>
-            {
+            SinkTypeIR::Partitioned(options) => {
                 let options = options.clone();
                 let input = lower_ir!(*input)?;
-                PhysNodeKind::PartitionedSink2 { input, options }
-            },
-
-            SinkTypeIR::Partitioned(PartitionedSinkOptionsIR {
-                base_path,
-                file_path_provider,
-                partition_strategy,
-                finish_callback,
-                file_format,
-                unified_sink_args,
-                max_rows_per_file,
-                approximate_bytes_per_file: _,
-            }) => {
-                // Convert to old parameters for now
-
-                let base_path = base_path.clone();
-                let file_path_cb = match file_path_provider {
-                    FileProviderType::Legacy(callback) => Some(callback.clone()),
-                    FileProviderType::Hive { .. } => None,
-                    FileProviderType::Function(_) => None,
-                };
-                let file_type = file_format.clone();
-                let finish_callback = finish_callback.clone();
-
-                let (variant, per_partition_sort_by) = match partition_strategy {
-                    PartitionStrategyIR::Keyed {
-                        keys,
-                        include_keys,
-                        keys_pre_grouped,
-                        per_partition_sort_by,
-                    } => {
-                        let v = if *keys_pre_grouped {
-                            PartitionVariantIR::Parted {
-                                key_exprs: keys.clone(),
-                                include_key: *include_keys,
-                            }
-                        } else {
-                            PartitionVariantIR::ByKey {
-                                key_exprs: keys.clone(),
-                                include_key: *include_keys,
-                            }
-                        };
-
-                        (v, per_partition_sort_by)
-                    },
-                    PartitionStrategyIR::FileSize => {
-                        (PartitionVariantIR::MaxSize(*max_rows_per_file), &vec![])
-                    },
-                };
-
-                let per_partition_sort_by = per_partition_sort_by.clone();
-
-                let UnifiedSinkArgs {
-                    mkdir,
-                    maintain_order,
-                    sync_on_close,
-                    cloud_options,
-                } = unified_sink_args.clone();
-
-                let sink_options = SinkOptions {
-                    sync_on_close,
-                    maintain_order,
-                    mkdir,
-                };
-
-                let mut input = lower_ir!(*input)?;
-                match &variant {
-                    PartitionVariantIR::MaxSize(_) => {},
-                    PartitionVariantIR::Parted {
-                        key_exprs,
-                        include_key: _,
-                    }
-                    | PartitionVariantIR::ByKey {
-                        key_exprs,
-                        include_key: _,
-                    } => {
-                        if key_exprs.is_empty() {
-                            polars_bail!(InvalidOperation: "cannot partition by-key without key expressions");
-                        }
-
-                        let input_schema = &phys_sm[input.node].output_schema;
-                        let mut select_output_schema = input_schema.as_ref().clone();
-                        for key_expr in key_exprs.iter() {
-                            select_output_schema.insert(
-                                key_expr.output_name().clone(),
-                                key_expr.dtype(input_schema.as_ref(), expr_arena)?.clone(),
-                            );
-                        }
-
-                        let select_output_schema = Arc::new(select_output_schema);
-                        let node = phys_sm.insert(PhysNode {
-                            output_schema: select_output_schema,
-                            kind: PhysNodeKind::Select {
-                                input,
-                                selectors: key_exprs.clone(),
-                                extend_original: true,
-                            },
-                        });
-                        input = PhysStream::first(node);
-                    },
-                };
-
-                PhysNodeKind::PartitionedSink {
-                    input,
-                    base_path: Arc::new(base_path),
-                    file_path_cb,
-                    sink_options,
-                    variant,
-                    file_type,
-                    cloud_options: cloud_options.map(Arc::unwrap_or_clone),
-                    per_partition_sort_by: (!per_partition_sort_by.is_empty())
-                        .then_some(per_partition_sort_by),
-                    finish_callback,
-                }
+                PhysNodeKind::PartitionedSink { input, options }
             },
         },
 
         IR::SinkMultiple { inputs } => {
+            disable_morsel_split.get_or_insert(true);
             let mut sinks = Vec::with_capacity(inputs.len());
             for input in inputs.clone() {
                 let phys_node_stream = match ir_arena.get(input) {
@@ -457,13 +346,14 @@ pub fn lower_ir(
                 let key_dtype = key_dtype.clone();
                 let mut expr = AExprBuilder::col(key.clone(), expr_arena);
                 if key_dtype.is_nested() {
-                    expr = expr.row_encode_unary(
+                    expr = AExprBuilder::row_encode(
+                        vec![expr.expr_ir(key_name.clone())],
+                        vec![key_dtype],
                         RowEncodingVariant::Ordered {
                             descending: None,
                             nulls_last: None,
                             broadcast_nulls: None,
                         },
-                        key_dtype,
                         expr_arena,
                     );
                 }
@@ -639,23 +529,31 @@ pub fn lower_ir(
 
             return Ok(stream);
         },
-
         IR::Union { inputs, options } => {
             let options = *options;
+
             let inputs = inputs
                 .clone() // Needed to borrow ir_arena mutably.
                 .into_iter()
                 .map(|input| lower_ir!(input))
                 .collect::<Result<_, _>>()?;
 
+            let kind = if options.maintain_order {
+                PhysNodeKind::OrderedUnion { inputs }
+            } else {
+                PhysNodeKind::UnorderedUnion { inputs }
+            };
+
             let node = phys_sm.insert(PhysNode {
                 output_schema,
-                kind: PhysNodeKind::OrderedUnion { inputs },
+                kind,
             });
             let mut stream = PhysStream::first(node);
+
             if let Some((offset, length)) = options.slice {
                 stream = build_slice_stream(stream, offset, length, phys_sm);
             }
+
             return Ok(stream);
         },
 
@@ -710,6 +608,7 @@ pub fn lower_ir(
                 // schema.
                 PhysNodeKind::InMemorySource {
                     df: Arc::new(DataFrame::empty_with_schema(output_schema.as_ref())),
+                    disable_morsel_split: disable_morsel_split.unwrap_or(true),
                 }
             } else if output_schema.is_empty()
                 && let Some((physical_rows, deleted_rows)) = unified_scan_args.row_count
@@ -730,6 +629,7 @@ pub fn lower_ir(
 
                 PhysNodeKind::InMemorySource {
                     df: Arc::new(DataFrame::empty_with_height(num_rows)),
+                    disable_morsel_split: disable_morsel_split.unwrap_or(true),
                 }
             } else {
                 let file_reader_builder: Arc<dyn FileReaderBuilder> = match &*scan_type {
@@ -744,25 +644,28 @@ pub fn lower_ir(
                             prefetch_limit: RelaxedCell::new_usize(0),
                             prefetch_semaphore: std::sync::OnceLock::new(),
                             shared_prefetch_wait_group_slot: Default::default(),
+                            io_metrics: std::sync::OnceLock::new(),
                         },
                     ) as _,
 
                     #[cfg(feature = "ipc")]
                     FileScanIR::Ipc {
-                        options: polars_io::ipc::IpcScanOptions {},
+                        options,
                         metadata: first_metadata,
                     } => Arc::new(crate::nodes::io_sources::ipc::builder::IpcReaderBuilder {
+                        options: Arc::new(options.clone()),
                         first_metadata: first_metadata.clone(),
                         prefetch_limit: RelaxedCell::new_usize(0),
                         prefetch_semaphore: std::sync::OnceLock::new(),
                         shared_prefetch_wait_group_slot: Default::default(),
+                        io_metrics: std::sync::OnceLock::new(),
                     }) as _,
 
                     #[cfg(feature = "csv")]
                     FileScanIR::Csv { options } => Arc::new(Arc::clone(options)) as _,
 
                     #[cfg(feature = "json")]
-                    FileScanIR::NDJson { options } => Arc::new(Arc::new(options.clone())) as _,
+                    FileScanIR::NDJson { options } => Arc::new(options.clone()) as _,
 
                     #[cfg(feature = "python")]
                     FileScanIR::PythonDataset {
@@ -782,7 +685,9 @@ pub fn lower_ir(
                     },
 
                     #[cfg(feature = "scan_lines")]
-                    FileScanIR::Lines { name: _ } => todo!(),
+                    FileScanIR::Lines { name: _ } => {
+                        Arc::new(crate::nodes::io_sources::lines::LineReaderBuilder {}) as _
+                    },
 
                     FileScanIR::Anonymous { .. } => todo!("unimplemented: AnonymousScan"),
                 };
@@ -837,6 +742,7 @@ pub fn lower_ir(
                     );
 
                     let pre_slice = unified_scan_args.pre_slice.clone();
+                    let disable_morsel_split = disable_morsel_split.unwrap_or(true);
 
                     let mut multi_scan_node = PhysNodeKind::MultiScan {
                         scan_sources,
@@ -859,6 +765,7 @@ pub fn lower_ir(
                         ),
                         table_statistics: unified_scan_args.table_statistics,
                         file_schema,
+                        disable_morsel_split,
                     };
 
                     let PhysNodeKind::MultiScan {
@@ -1009,6 +916,7 @@ pub fn lower_ir(
                     // Fallback to in-memory engine.
                     let input = PhysNodeKind::InMemorySource {
                         df: Arc::new(DataFrame::default()),
+                        disable_morsel_split: disable_morsel_split.unwrap_or(true),
                     };
                     let input_key =
                         phys_sm.insert(PhysNode::new(Arc::new(Schema::default()), input));
@@ -1089,7 +997,8 @@ pub fn lower_ir(
                 &keys,
                 expr_arena,
                 input_schema,
-            );
+            )
+            .is_some();
 
             return build_group_by_stream(
                 phys_input,
@@ -1116,17 +1025,42 @@ pub fn lower_ir(
         } => {
             let input_left = *input_left;
             let input_right = *input_right;
+            let input_left_schema = IR::schema_with_cache(input_left, ir_arena, schema_cache);
+            let input_right_schema = IR::schema_with_cache(input_right, ir_arena, schema_cache);
             let left_on = left_on.clone();
             let right_on = right_on.clone();
+            let get_expr_name = |e: &ExprIR| e.output_name().clone();
+            let left_on_names = left_on.iter().map(get_expr_name).collect_vec();
+            let right_on_names = right_on.iter().map(get_expr_name).collect_vec();
             let args = options.args.clone();
             let options = options.options.clone();
+            let left_df_sortedness = is_sorted(input_left, ir_arena, expr_arena);
+            let left_on_sorted = are_keys_sorted_any(
+                left_df_sortedness.as_ref(),
+                &left_on,
+                expr_arena,
+                &input_left_schema,
+            );
+            let right_df_sortedness = is_sorted(input_right, ir_arena, expr_arena);
+            let right_on_sorted = are_keys_sorted_any(
+                right_df_sortedness.as_ref(),
+                &right_on,
+                expr_arena,
+                &input_right_schema,
+            );
+            let join_keys_sorted_together =
+                Option::zip(left_on_sorted.as_ref(), right_on_sorted.as_ref())
+                    .is_some_and(|(ls, rs)| ls == rs);
+
             let phys_left = lower_ir!(input_left)?;
             let phys_right = lower_ir!(input_right)?;
+
             if (args.how.is_equi() || args.how.is_semi_anti()) && !args.validation.needs_checks() {
                 // When lowering the expressions for the keys we need to ensure we keep around the
                 // payload columns, otherwise the input nodes can get replaced by input-independent
                 // nodes since the lowering code does not see we access any non-literal expressions.
                 // So we add dummy expressions before lowering and remove them afterwards.
+
                 let mut aug_left_on = left_on.clone();
                 for name in phys_sm[phys_left.node].output_schema.iter_names() {
                     let col_expr = expr_arena.add(AExpr::Column(name.clone()));
@@ -1137,7 +1071,8 @@ pub fn lower_ir(
                     let col_expr = expr_arena.add(AExpr::Column(name.clone()));
                     aug_right_on.push(ExprIR::new(col_expr, OutputName::ColumnLhs(name.clone())));
                 }
-                let (trans_input_left, mut trans_left_on) = lower_exprs(
+
+                let (mut trans_input_left, mut trans_left_on) = lower_exprs(
                     phys_left,
                     &aug_left_on,
                     expr_arena,
@@ -1145,7 +1080,7 @@ pub fn lower_ir(
                     expr_cache,
                     ctx,
                 )?;
-                let (trans_input_right, mut trans_right_on) = lower_exprs(
+                let (mut trans_input_right, mut trans_right_on) = lower_exprs(
                     phys_right,
                     &aug_right_on,
                     expr_arena,
@@ -1153,10 +1088,112 @@ pub fn lower_ir(
                     expr_cache,
                     ctx,
                 )?;
+
                 trans_left_on.drain(left_on.len()..);
                 trans_right_on.drain(right_on.len()..);
 
-                let node = if args.how.is_equi() {
+                let node = if args.how.is_equi() && join_keys_sorted_together {
+                    // For merge-joins, evaluate key expressions if they are non-trivial,
+                    // row-encode them if there are multiple, and append them as new columns
+                    // to the input dataframes.
+                    let row_encode_key_cols = left_on_names.len() > 1;
+                    let mut left_key_col_descending =
+                        left_on_sorted.as_ref().unwrap()[0].descending.unwrap();
+                    let mut right_key_col_descending =
+                        right_on_sorted.as_ref().unwrap()[0].descending.unwrap();
+                    let mut left_key_col_nulls_last =
+                        left_on_sorted.as_ref().unwrap()[0].nulls_last.unwrap();
+                    let mut right_key_col_nulls_last =
+                        right_on_sorted.as_ref().unwrap()[0].nulls_last.unwrap();
+                    for (
+                        on,
+                        trans_on,
+                        trans_input,
+                        phys,
+                        expr_sorted,
+                        descending,
+                        nulls_last,
+                        side_schema,
+                    ) in [
+                        (
+                            left_on,
+                            &mut trans_left_on,
+                            &mut trans_input_left,
+                            phys_left,
+                            &left_on_sorted.unwrap(),
+                            &mut left_key_col_descending,
+                            &mut left_key_col_nulls_last,
+                            &input_left_schema,
+                        ),
+                        (
+                            right_on,
+                            &mut trans_right_on,
+                            &mut trans_input_right,
+                            phys_right,
+                            &right_on_sorted.unwrap(),
+                            &mut right_key_col_descending,
+                            &mut right_key_col_nulls_last,
+                            &input_right_schema,
+                        ),
+                    ] {
+                        let expr_is_trivial =
+                            |c: &ExprIR| matches!(expr_arena.get(c.node()), AExpr::Column(_));
+                        if row_encode_key_cols || !expr_is_trivial(&on[0]) {
+                            let sorted_descending = expr_sorted
+                                .iter()
+                                .map(|s| s.descending.unwrap())
+                                .collect_vec();
+                            let sorted_nulls_last = expr_sorted
+                                .iter()
+                                .map(|s| s.nulls_last.unwrap())
+                                .collect_vec();
+
+                            if row_encode_key_cols {
+                                let tfc = ToFieldContext::new(expr_arena, side_schema);
+                                let expr_dtype =
+                                    |e: &ExprIR| expr_arena.get(e.node()).to_dtype(&tfc);
+                                let nulls_last_encoded = sorted_nulls_last[0];
+                                let row_encode_col_expr = AExprBuilder::row_encode(
+                                    trans_on.clone(),
+                                    trans_on.iter().map(expr_dtype).try_collect_vec()?,
+                                    RowEncodingVariant::Ordered {
+                                        descending: Some(sorted_descending),
+                                        nulls_last: Some(sorted_nulls_last),
+                                        broadcast_nulls: Some(!args.nulls_equal),
+                                    },
+                                    expr_arena,
+                                )
+                                .expr_ir(merge_join::KEY_COL_NAME);
+                                trans_on.clear();
+                                trans_on.push(row_encode_col_expr);
+                                *descending = false;
+                                *nulls_last = nulls_last_encoded;
+                            } else {
+                                trans_on[0] = trans_on[0]
+                                    .with_alias(PlSmallStr::from_str(merge_join::KEY_COL_NAME));
+                            }
+                            *trans_input = build_hstack_stream(
+                                phys, trans_on, expr_arena, phys_sm, expr_cache, ctx,
+                            )?;
+                        }
+                    }
+                    assert!(left_key_col_descending == right_key_col_descending);
+                    assert!(left_key_col_nulls_last == right_key_col_nulls_last);
+
+                    phys_sm.insert(PhysNode::new(
+                        output_schema,
+                        PhysNodeKind::MergeJoin {
+                            input_left: trans_input_left,
+                            input_right: trans_input_right,
+                            left_on: left_on_names,
+                            right_on: right_on_names,
+                            descending: left_key_col_descending,
+                            nulls_last: left_key_col_nulls_last,
+                            keys_row_encoded: row_encode_key_cols,
+                            args: args.clone(),
+                        },
+                    ))
+                } else if args.how.is_equi() {
                     phys_sm.insert(PhysNode::new(
                         output_schema,
                         PhysNodeKind::EquiJoin {
@@ -1323,7 +1360,8 @@ pub fn lower_ir(
                 &keys,
                 expr_arena,
                 input_schema,
-            );
+            )
+            .is_some();
 
             let mut stream = build_group_by_stream(
                 phys_input,
