@@ -8,6 +8,7 @@
 pub mod abs_diff;
 pub mod algebraic_ops;
 pub mod arena;
+pub mod array;
 pub mod binary_search;
 pub mod cache;
 pub mod cardinality_sketch;
@@ -18,15 +19,17 @@ mod config;
 pub mod small_bytes;
 pub use config::check_allow_importing_interval_as_struct;
 pub mod arg_min_max;
+pub mod bool;
 pub mod cpuid;
 pub mod error;
 pub mod float16;
 pub mod floor_divmod;
 pub mod functions;
 pub mod hashing;
+pub mod ideal_morsel_size;
 pub mod idx_map;
-pub mod idx_mapper;
 pub mod idx_vec;
+pub mod live_timer;
 pub mod marked_usize;
 pub mod mem;
 pub mod min_max;
@@ -35,7 +38,6 @@ pub mod parma;
 pub mod pl_path;
 mod pl_ref_str;
 pub mod pl_str;
-mod plpath;
 pub mod priority;
 pub mod regex_cache;
 pub mod relaxed_cell;
@@ -52,7 +54,6 @@ pub mod sys;
 pub mod total_ord;
 pub mod unique_id;
 pub mod with_drop;
-
 pub use functions::*;
 pub mod compression;
 pub mod file;
