@@ -19,5 +19,5 @@ use polars::prelude::*;
 pub(crate) fn create_df() -> DataFrame {
     let s0 = Column::new("days".into(), [0, 1, 2, 3, 4].as_ref());
     let s1 = Column::new("temp".into(), [22.1, 19.9, 7., 2., 3.].as_ref());
-    DataFrame::new(vec![s0, s1]).unwrap()
+    DataFrame::new_infer_height(vec![s0, s1]).unwrap()
 }
