@@ -2,7 +2,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     // --8<-- [start:example]
     use polars::prelude::*;
 
-    let q = LazyCsvReader::new(PlPath::new("docs/assets/data/iris.csv"))
+    let q = LazyCsvReader::new(PlRefPath::new("docs/assets/data/iris.csv"))
         .with_has_header(true)
         .finish()?
         .filter(col("sepal_length").gt(lit(5)))
