@@ -1,22 +1,24 @@
 from __future__ import annotations
 
 import os
+import pickle
 import warnings
 from datetime import datetime, timezone
 from pathlib import Path
 from typing import Any
 
 import pyarrow as pa
-import pyarrow.fs
 import pytest
 from deltalake import DeltaTable, write_deltalake
 from deltalake.exceptions import DeltaError, TableNotFoundError
 from deltalake.table import TableMerger
 
 import polars as pl
+from polars.io.cloud._utils import NoPickleOption
 from polars.io.cloud.credential_provider._builder import (
     _init_credential_provider_builder,
 )
+from polars.io.delta._dataset import DeltaDataset
 from polars.testing import assert_frame_equal, assert_frame_not_equal
 
 
@@ -25,6 +27,20 @@ def delta_table_path(io_files_path: Path) -> Path:
     return io_files_path / "delta-table"
 
 
+def new_pl_delta_dataset(source: str | DeltaTable) -> DeltaDataset:
+    return DeltaDataset(
+        table_=NoPickleOption(source if isinstance(source, DeltaTable) else None),
+        table_uri_=source if not isinstance(source, DeltaTable) else None,
+        version=None,
+        storage_options=None,
+        credential_provider_builder=None,
+        delta_table_options=None,
+        use_pyarrow=False,
+        pyarrow_options=None,
+        rechunk=False,
+    )
+
+
 def test_scan_delta(delta_table_path: Path) -> None:
     ldf = pl.scan_delta(delta_table_path, version=0)
 
@@ -74,6 +90,43 @@ def test_scan_delta_columns(delta_table_path: Path) -> None:
     assert_frame_equal(expected, ldf.collect(), check_dtypes=False)
 
 
+def test_scan_delta_polars_storage_options_keys(
+    delta_table_path: Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capfd: pytest.CaptureFixture[str],
+) -> None:
+    monkeypatch.setenv("POLARS_VERBOSE_SENSITIVE", "1")
+    lf = pl.scan_delta(
+        delta_table_path,
+        version=0,
+        storage_options={
+            "file_cache_ttl": 7,
+            "max_retries": 3,
+            "retry_timeout_ms": 9873,
+            "retry_init_backoff_ms": 9874,
+            "retry_max_backoff_ms": 9875,
+            "retry_base_multiplier": 3.14159,
+        },
+        credential_provider=None,
+    ).select("name")
+
+    lf.collect()
+
+    capture = capfd.readouterr().err
+
+    assert "file_cache_ttl: 7" in capture
+
+    assert (
+        """\
+max_retries: Some(3), \
+retry_timeout: Some(9.873s), \
+retry_init_backoff: Some(9.874s), \
+retry_max_backoff: Some(9.875s), \
+retry_base_multiplier: Some(TotalOrdWrap(3.14159)) }"""
+        in capture
+    )
+
+
 def test_scan_delta_relative(delta_table_path: Path) -> None:
     rel_delta_table_path = str(delta_table_path / ".." / "delta-table")
 
@@ -488,7 +541,7 @@ def test_write_delta_w_compatible_schema(series: pl.Series, tmp_path: Path) -> N
     "expr",
     [
         pl.datetime(2010, 1, 1, time_unit="us", time_zone="UTC"),
-        pl.datetime(2010, 1, 1, time_unit="ns", time_zone="EST"),
+        pl.datetime(2010, 1, 1, time_unit="ns", time_zone="America/New_York"),
         pl.datetime(2010, 1, 1, time_unit="ms", time_zone="Europe/Amsterdam"),
     ],
 )
@@ -634,7 +687,18 @@ def test_scan_delta_nanosecond_timestamp(
 
     root = tmp_path / "delta"
 
-    df.write_delta(root)
+    import deltalake
+
+    df.write_delta(
+        root,
+        delta_write_options={
+            "writer_properties": deltalake.WriterProperties(
+                default_column_properties=deltalake.ColumnProperties(
+                    statistics_enabled="NONE"
+                )
+            )
+        },
+    )
 
     # Manually overwrite the file with one that has nanosecond timestamps.
     parquet_files = [x for x in root.iterdir() if x.suffix == ".parquet"]
@@ -759,7 +823,7 @@ def test_scan_delta_schema_evolution_nested_struct_field_19915(tmp_path: Path) -
 def test_scan_delta_storage_options_from_delta_table(
     tmp_path: Path, monkeypatch: pytest.MonkeyPatch
 ) -> None:
-    import polars.io.delta
+    import polars.io.delta._dataset
 
     storage_options_checked = False
 
@@ -779,7 +843,7 @@ def test_scan_delta_storage_options_from_delta_table(
         return pl.scan_parquet(*a, **kw)
 
     monkeypatch.setattr(
-        polars.io.delta, "scan_parquet", assert_scan_parquet_storage_options
+        polars.io.delta._dataset, "scan_parquet", assert_scan_parquet_storage_options
     )
 
     df = pl.DataFrame({"a": ["test"], "properties": [{"property_key": {"item": 1}}]})
@@ -848,7 +912,317 @@ endpoint_url = http://127.0.0.1:54321
     }
 
     with pytest.raises((DeltaError, OSError), match=r"http://127.0.0.1:54321"):
-        pl.scan_delta("s3://.../...")
+        pl.scan_delta("s3://.../...").collect()
 
     with pytest.raises((DeltaError, OSError), match=r"http://127.0.0.1:54321"):
         pl.DataFrame({"x": 1}).write_delta("s3://.../...", mode="append")
+
+
+# TODO: uncomment dtype when fixed
+@pytest.mark.parametrize(
+    "expr",
+    [
+        # Bool
+        # pl.col.bool == False,  ## see github issue #26290, to be confirmed
+        # pl.col.bool <= False,
+        # pl.col.bool < True,
+        pl.col.bool.is_null(),
+        # Integer
+        pl.col.int == 2,
+        pl.col.int <= 2,
+        pl.col.int < 3,
+        pl.col.int.is_null(),
+        (pl.col.int < 2) & (pl.col.int.is_not_null()),
+        # Float ## see github issue #26238
+        # pl.col.float == 2.0,
+        # pl.col.float <= 2.0,
+        # pl.col.float < 3.0,
+        pl.col.float.is_null(),
+        # mixed
+        (pl.col.int == 2) & (pl.col.float.is_not_null()),
+        # String
+        pl.col.string == "b",
+        pl.col.string <= "b",
+        pl.col.string.is_null(),
+        # Decimal
+        pl.col.decimal == pl.lit(2.0).cast(pl.Decimal(10, 2)),
+        pl.col.decimal <= pl.lit(2.0).cast(pl.Decimal(10, 2)),
+        pl.col.decimal < pl.lit(3.0).cast(pl.Decimal(10, 2)),
+        pl.col.decimal.is_null(),
+        # Struct # see github issue #26239
+        # pl.col.struct == {"x": 2, "y": 20},
+        # pl.col.struct.is_null(),
+        # Date & datetime
+        pl.col.date == pl.date(2020, 1, 1),
+        pl.col.datetime == pl.datetime(2020, 1, 1),
+        # on predicate
+        pl.col.p == 10,
+    ],
+)
+@pytest.mark.write_disk
+def test_parquet_filter_on_file_statistics_23780(
+    tmp_path: Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capfd: pytest.CaptureFixture[str],
+    expr: pl.Expr,
+) -> None:
+    df = pl.DataFrame(
+        {
+            "p": [10, 10, 20, 20, 30, 30],
+            "a": [1, 2, 3, 4, 5, None],
+            "bool": [False, False, True, True, True, None],
+            "int": [1, 2, 3, 4, 5, None],
+            "float": [1.0, 2.0, 3.0, 4.0, 5.0, None],
+            "string": ["a", "b", "c", "cc", "ccc", None],
+            "struct": [
+                {"x": 1, "y": 10},
+                {"x": 2, "y": 20},
+                {"x": 3, "y": 30},
+                {"x": 4, "y": 40},
+                {"x": 5, "y": 50},
+                None,
+            ],
+        }
+    ).with_columns(
+        decimal=pl.col.a.cast(pl.Decimal(10, 2)),
+        date=pl.date_range(pl.date(2020, 1, 1), pl.date(2020, 1, 6), closed="both"),
+        datetime=pl.datetime_range(
+            pl.datetime(2020, 1, 1), pl.datetime(2020, 1, 6), closed="both"
+        ),
+    )
+    root = tmp_path / "delta"
+
+    df.write_delta(root, delta_write_options={"partition_by": "p"})
+
+    monkeypatch.setenv("POLARS_VERBOSE", "1")
+    capfd.readouterr()
+
+    assert_frame_equal(
+        pl.scan_delta(root).filter(expr).collect(),
+        df.filter(expr),
+        check_column_order=False,
+        check_row_order=False,
+    )
+    assert "skipping 2 / 3 files" in capfd.readouterr().err
+
+
+@pytest.mark.parametrize(
+    ("expr", "n_cols", "expect_n_files_skipped"),
+    [
+        (pl.col.a == 2, "0", 0),
+        (pl.col.a == 2, "1", 1),
+        (pl.col.b == 2, "1", 0),
+        (pl.col.a == 2, "2", 1),
+    ],
+)
+@pytest.mark.write_disk
+def test_parquet_filter_on_file_statistics_partial_23780(
+    tmp_path: Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capfd: pytest.CaptureFixture[str],
+    expr: pl.Expr,
+    n_cols: str,
+    expect_n_files_skipped: int,
+) -> None:
+    df = pl.DataFrame({"p": [10, 10, 20, 20], "a": [1, 2, 3, 4], "b": [1, 2, 3, 4]})
+
+    root = tmp_path / "delta"
+    df.write_delta(
+        root,
+        delta_write_options={
+            "partition_by": "p",
+            "configuration": {
+                "delta.dataSkippingNumIndexedCols": n_cols  # Disable stats collection
+            },
+        },
+    )
+
+    monkeypatch.setenv("POLARS_VERBOSE", "1")
+    capfd.readouterr()
+
+    assert_frame_equal(
+        pl.scan_delta(root).filter(expr).collect(),
+        df.filter(expr),
+        check_column_order=False,
+        check_row_order=False,
+    )
+    assert f"skipping {expect_n_files_skipped} / 2 files" in capfd.readouterr().err
+
+
+@pytest.mark.write_disk
+def test_parquet_filter_on_file_statistics_delete_partition_23780(
+    tmp_path: Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capfd: pytest.CaptureFixture[str],
+) -> None:
+    df = pl.DataFrame(
+        {
+            "p": [10, 10, 20, 30],
+            "a": [1, 2, 3, 4],
+        }
+    )
+    root = tmp_path / "delta"
+
+    df.write_delta(root, delta_write_options={"partition_by": "p"})
+
+    monkeypatch.setenv("POLARS_VERBOSE", "1")
+    capfd.readouterr()
+
+    expr = pl.col.a == 2
+    assert_frame_equal(
+        pl.scan_delta(root).filter(expr).collect(),
+        df.filter(expr),
+        check_column_order=False,
+        check_row_order=False,
+    )
+    assert "skipping 2 / 3 files" in capfd.readouterr().err
+
+    from deltalake import DeltaTable
+
+    dt = DeltaTable(root)
+    dt.delete("p = 30")
+
+    assert_frame_equal(
+        pl.scan_delta(root).filter(expr).collect(),
+        df.filter(expr),
+        check_column_order=False,
+        check_row_order=False,
+    )
+    assert "skipping 1 / 2 files" in capfd.readouterr().err
+
+
+@pytest.mark.parametrize("use_pyarrow", [True, False])
+@pytest.mark.write_disk
+def test_scan_delta_use_pyarrow(tmp_path: Path, use_pyarrow: bool) -> None:
+    df = pl.DataFrame({"year": [2025, 2026, 2026], "month": [0, 0, 0]})
+    df.write_delta(tmp_path, delta_write_options={"partition_by": "year"})
+
+    assert_frame_equal(
+        pl.scan_delta(tmp_path, use_pyarrow=use_pyarrow)
+        .filter(pl.col("year") == 2026)
+        .collect(),
+        pl.DataFrame({"year": [2026, 2026], "month": [0, 0]}),
+    )
+
+    assert_frame_equal(
+        pl.scan_delta(tmp_path, use_pyarrow=use_pyarrow)
+        .filter(pl.col("year") == 2026)
+        .head(1)
+        .collect(),
+        pl.DataFrame({"year": [2026], "month": [0]}),
+    )
+
+    # Delta does not have stable file scan ordering.
+    assert (
+        pl.scan_delta(tmp_path, use_pyarrow=use_pyarrow).head(1).collect().height == 1
+    )
+
+
+@pytest.mark.parametrize("use_pyarrow", [True, False])
+@pytest.mark.write_disk
+def test_scan_delta_use_pyarrow_single_file(tmp_path: Path, use_pyarrow: bool) -> None:
+    df = pl.DataFrame({"year": [2025, 2026, 2026], "month": [0, 0, 0]})
+    df.write_delta(tmp_path)
+
+    assert_frame_equal(
+        pl.scan_delta(tmp_path, use_pyarrow=use_pyarrow)
+        .filter(pl.col("year") == 2026)
+        .collect(),
+        pl.DataFrame({"year": [2026, 2026], "month": [0, 0]}),
+    )
+
+    assert_frame_equal(
+        pl.scan_delta(tmp_path, use_pyarrow=use_pyarrow)
+        .filter(pl.col("year") == 2026)
+        .head(1)
+        .collect(),
+        pl.DataFrame({"year": [2026], "month": [0]}),
+    )
+
+    assert_frame_equal(
+        pl.scan_delta(tmp_path, use_pyarrow=use_pyarrow).head(1).collect(),
+        pl.DataFrame({"year": [2025], "month": [0]}),
+    )
+
+    assert_frame_equal(
+        pl.scan_delta(tmp_path, use_pyarrow=use_pyarrow)
+        .head(1)
+        .filter(pl.col("year") == 2026)
+        .collect(),
+        pl.DataFrame(schema={"year": pl.Int64, "month": pl.Int64}),
+    )
+
+
+@pytest.mark.write_disk
+def test_delta_dataset_does_not_pickle_table_object(tmp_path: Path) -> None:
+    df = pl.DataFrame({"row_index": [0, 1, 2, 3, 4]})
+    df.write_delta(tmp_path)
+
+    dataset = new_pl_delta_dataset(DeltaTable(tmp_path))
+
+    assert dataset.table_.get() is not None
+    dataset = pickle.loads(pickle.dumps(dataset))
+    assert dataset.table_.get() is None
+
+    assert_frame_equal(dataset.to_dataset_scan()[0].collect(), df)  # type: ignore[index]
+
+
+@pytest.mark.parametrize("use_pyarrow", [True, False])
+@pytest.mark.write_disk
+def test_delta_partition_filter(tmp_path: Path, use_pyarrow: bool) -> None:
+    df = pl.DataFrame({"row_index": [0, 1, 2, 3, 4], "year": 2026})
+    df.write_delta(tmp_path, delta_write_options={"partition_by": "year"})
+
+    for path in DeltaTable(tmp_path).file_uris():
+        Path(path).unlink()
+
+    with pytest.raises((FileNotFoundError, OSError)):
+        pl.scan_delta(tmp_path, use_pyarrow=use_pyarrow).collect()
+
+    assert_frame_equal(
+        pl.scan_delta(tmp_path, use_pyarrow=use_pyarrow)
+        .filter(pl.col("year") < 0)
+        .collect(),
+        pl.DataFrame(schema=df.schema),
+    )
+
+
+@pytest.mark.write_disk
+@pytest.mark.parametrize("use_pyarrow", [True, False])
+def test_scan_delta_collect_without_version_scans_latest(
+    tmp_path: Path,
+    use_pyarrow: bool,
+    monkeypatch: pytest.MonkeyPatch,
+    capfd: pytest.CaptureFixture[str],
+) -> None:
+    pl.DataFrame({"a": [0]}).write_delta(tmp_path)
+    table = DeltaTable(tmp_path)
+
+    q = pl.scan_delta(table, use_pyarrow=use_pyarrow)
+
+    assert_frame_equal(q.collect(), pl.DataFrame({"a": [0]}))
+
+    pl.DataFrame({"a": [1]}).write_delta(table, mode="append")
+
+    assert_frame_equal(q.collect().sort("*"), pl.DataFrame({"a": [0, 1]}))
+
+    version = table.version()
+
+    q_with_id = pl.scan_delta(table, use_pyarrow=use_pyarrow, version=version)
+
+    assert_frame_equal(q_with_id.collect().sort("*"), pl.DataFrame({"a": [0, 1]}))
+
+    pl.DataFrame({"a": [2]}).write_delta(table, mode="append")
+
+    assert_frame_equal(q.collect().sort("*"), pl.DataFrame({"a": [0, 1, 2]}))
+
+    monkeypatch.setenv("POLARS_VERBOSE", "1")
+    capfd.readouterr()
+
+    assert_frame_equal(q_with_id.collect().sort("*"), pl.DataFrame({"a": [0, 1]}))
+
+    capture = capfd.readouterr().err
+
+    assert (
+        "DeltaDataset: to_dataset_scan(): early return (version_key = '1')" in capture
+    )
