@@ -13,7 +13,9 @@ from typing import TYPE_CHECKING, Any
 import pytest
 
 import polars as pl
+from polars.exceptions import ComputeError
 from polars.testing.asserts.frame import assert_frame_equal
+from tests.unit.io.conftest import format_file_uri, normalize_path_separator_pl
 
 if TYPE_CHECKING:
     from collections.abc import Callable
@@ -616,7 +618,6 @@ def test_scan_nonexistent_path(format: str) -> None:
     "streaming",
     [True, False],
 )
-@pytest.mark.skipif(sys.platform == "win32", reason="Windows paths are different")
 def test_scan_include_file_paths(
     tmp_path: Path,
     scan_func: Callable[..., pl.LazyFrame],
@@ -631,7 +632,7 @@ def test_scan_include_file_paths(
         dfs.append(pl.DataFrame({"x": 10 * [x]}).with_columns(path=pl.lit(str(path))))
         write_func(dfs[-1].drop("path"), path)
 
-    df = pl.concat(dfs)
+    df = pl.concat(dfs).with_columns(normalize_path_separator_pl(pl.col("path")))
     assert df.columns == ["x", "path"]
 
     with pytest.raises(
@@ -1026,7 +1027,6 @@ def test_only_project_missing(scan_type: tuple[Any, Any]) -> None:
     )
 
 
-@pytest.mark.skipif(sys.platform == "win32", reason="windows paths are a mess")
 @pytest.mark.write_disk
 @pytest.mark.parametrize(
     "scan_type",
@@ -1045,13 +1045,19 @@ def test_async_read_21945(tmp_path: Path, scan_type: tuple[Any, Any]) -> None:
     pl.DataFrame({"value": [3]}).write_parquet(f2)
 
     df = (
-        pl.scan_parquet(["file://" + str(f1), str(f2)], include_file_paths="foo")
+        pl.scan_parquet([format_file_uri(str(f1)), str(f2)], include_file_paths="foo")
         .filter(value=1)
         .collect()
     )
 
     assert_frame_equal(
-        df, pl.DataFrame({"value": [1], "foo": ["file://" + f1.as_posix()]})
+        df,
+        pl.DataFrame(
+            {
+                "value": [1],
+                "foo": [format_file_uri(f1)],
+            }
+        ),
     )
 
 
@@ -1150,13 +1156,6 @@ def test_scan_empty_paths_friendly_error(
     assert "glob: true)." in exc_str
     assert exc_str.count(tmp_path.name) == 1
 
-    assert (
-        name_in_context
-        in exc_str.split(
-            "This error occurred with the following context stack:", maxsplit=1
-        )[1]
-    )
-
     if scan_function is pl.scan_parquet:
         assert (
             "Hint: passing a schema can allow this scan to succeed with an empty DataFrame."
@@ -1223,13 +1222,44 @@ def test_scan_with_schema_skips_schema_inference(
     assert_frame_equal(q.collect(engine="streaming"), pl.DataFrame(schema=schema))
 
 
-@pytest.fixture(scope="session")
-def corrupt_compressed_csv() -> bytes:
-    large_and_simple_csv = b"line_val\n" * 200_000
-    compressed_data = zlib.compress(large_and_simple_csv, level=0)
+@pytest.mark.parametrize("format_name", ["csv", "ndjson", "lines"])
+def test_scan_negative_slice_decompress(format_name: str) -> None:
+    col_name = "x"
+
+    # We could go through sink, but not for lines and this way we are testing
+    # more narrowly.
+    def format_line(val: int) -> str:
+        if format_name == "csv":
+            return f"{val}\n"
+        elif format_name == "ndjson":
+            return f'{{"{col_name}":{val}}}\n'
+        elif format_name == "lines":
+            return f"{val}\n"
+        else:
+            pytest.fail("unreachable")
+
+    buf = bytearray()
+    if format_name == "csv":
+        buf.extend(f"{col_name}\n".encode())
+    for val in range(47):
+        buf.extend(format_line(val).encode())
+
+    compressed_data = zlib.compress(buf, level=1)
+
+    lf = getattr(pl, f"scan_{format_name}")(compressed_data).slice(-9, 5)
+    if format_name == "lines":
+        lf = lf.select(pl.col("lines").alias(col_name).str.to_integer())
+
+    expected = [pl.Series("x", [38, 39, 40, 41, 42])]
+    got = lf.collect(engine="streaming")
+    assert_frame_equal(got, pl.DataFrame(expected))
+
+
+def corrupt_compressed_impl(base_line: bytes, target_size: int) -> bytes:
+    large_and_simple_data = base_line * round(target_size / len(base_line))
+    compressed_data = zlib.compress(large_and_simple_data, level=0)
 
     corruption_start_pos = round(len(compressed_data) * 0.9)
-    assert corruption_start_pos > 1_600_000
     corruption_len = 500
 
     # The idea is to corrupt the input to make sure the scan never fully
@@ -1238,11 +1268,24 @@ def corrupt_compressed_csv() -> bytes:
     corrupted_data[corruption_start_pos : corruption_start_pos + corruption_len] = (
         b"\00"
     )
-    # ~1.6MB of valid zlib compressed CSV to read before the corrupted data
+    # ~1.8MB of valid zlib compressed data to read before the corrupted data
     # appears.
     return corrupted_data
 
 
+@pytest.fixture(scope="session")
+def corrupt_compressed_csv() -> bytes:
+    return corrupt_compressed_impl(b"line_val\n", int(2e6))
+
+
+@pytest.fixture(scope="session")
+def corrupt_compressed_ndjson() -> bytes:
+    # The decompressor is part of the line-batch provider which only stops once
+    # processing noticed that it doesn't need anymore data. This can take a
+    # bit. Tested with debug/release and calm/stressed machine.
+    return corrupt_compressed_impl(b'{"line_val": 45}\n', int(100e6))
+
+
 @pytest.mark.parametrize("schema", [{"line_val": pl.String}, None])
 def test_scan_csv_streaming_decompression(
     corrupt_compressed_csv: bytes, schema: Any
@@ -1255,9 +1298,74 @@ def test_scan_csv_streaming_decompression(
         .collect(engine="streaming")
     )
 
-    expected = pl.DataFrame(
-        [
-            pl.Series("line_val", ["line_val"] * slice_count, dtype=pl.String),
-        ]
+    expected = [
+        pl.Series("line_val", ["line_val"] * slice_count, dtype=pl.String),
+    ]
+    assert_frame_equal(df, pl.DataFrame(expected))
+
+
+@pytest.mark.slow
+@pytest.mark.parametrize("schema", [{"line_val": pl.Int64}, None])
+def test_scan_ndjson_streaming_decompression(
+    corrupt_compressed_ndjson: bytes, schema: Any
+) -> None:
+    slice_count = 11
+
+    df = (
+        pl.scan_ndjson(io.BytesIO(corrupt_compressed_ndjson), schema=schema)
+        .slice(0, slice_count)
+        .collect(engine="streaming")
+    )
+
+    expected = [
+        pl.Series("line_val", [45] * slice_count, dtype=pl.Int64),
+    ]
+    assert_frame_equal(df, pl.DataFrame(expected))
+
+
+def test_scan_file_uri_hostname_component() -> None:
+    q = pl.scan_parquet("file://hostname:80/data.parquet")
+
+    with pytest.raises(
+        ComputeError,
+        match="unsupported: non-empty hostname for 'file:' URI: 'hostname:80'",
+    ):
+        q.collect()
+
+
+@pytest.mark.write_disk
+@pytest.mark.parametrize("polars_force_async", ["0", "1"])
+@pytest.mark.parametrize("n_repeats", [1, 2])
+def test_scan_path_expansion_sorting_24528(
+    tmp_path: Path,
+    polars_force_async: str,
+    n_repeats: int,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
+    monkeypatch.setenv("POLARS_FORCE_ASYNC", polars_force_async)
+
+    relpaths = ["a.parquet", "a/a.parquet", "ab.parquet"]
+
+    for p in relpaths:
+        Path(tmp_path / p).parent.mkdir(exist_ok=True, parents=True)
+        pl.DataFrame({"relpath": p}).write_parquet(tmp_path / p)
+
+    assert_frame_equal(
+        pl.scan_parquet(n_repeats * [tmp_path]).collect(),
+        pl.DataFrame({"relpath": n_repeats * relpaths}),
+    )
+
+    assert_frame_equal(
+        pl.scan_parquet(n_repeats * [f"{tmp_path}/**/*"]).collect(),
+        pl.DataFrame({"relpath": n_repeats * relpaths}),
+    )
+
+    assert_frame_equal(
+        pl.scan_parquet(n_repeats * [format_file_uri(tmp_path) + "/"]).collect(),
+        pl.DataFrame({"relpath": n_repeats * relpaths}),
+    )
+
+    assert_frame_equal(
+        pl.scan_parquet(n_repeats * [format_file_uri(f"{tmp_path}/**/*")]).collect(),
+        pl.DataFrame({"relpath": n_repeats * relpaths}),
     )
-    assert_frame_equal(df, expected)
