@@ -636,7 +636,7 @@ def test_schema_overrides(path_xlsx: Path, path_xlsb: Path, path_ods: Path) -> N
     )
     df = pl.read_excel(
         path_xlsx,
-        sheet_name=["test4", "test4"],
+        sheet_name=("test4", "test4"),
         schema_overrides=overrides,
     )
     for col, dtype in overrides.items():
