@@ -2629,12 +2629,30 @@ def test_group_bool_unique_25267(maintain_order: bool, stable: bool) -> None:
 
 
 @pytest.mark.parametrize("group_as_slice", [False, True])
-@pytest.mark.parametrize("n", [10, 100, 1_000, 10_000])
+@pytest.mark.parametrize("n", [10, 100, 519])
 @pytest.mark.parametrize(
     "dtype", [pl.Int32, pl.Boolean, pl.String, pl.Categorical, pl.List(pl.Int32)]
 )
 def test_group_by_first_last(
     group_as_slice: bool, n: int, dtype: PolarsDataType
+) -> None:
+    group_by_first_last_test_impl(group_as_slice, n, dtype)
+
+
+@pytest.mark.slow
+@pytest.mark.parametrize("group_as_slice", [False, True])
+@pytest.mark.parametrize("n", [1056, 10_432])
+@pytest.mark.parametrize(
+    "dtype", [pl.Int32, pl.Boolean, pl.String, pl.Categorical, pl.List(pl.Int32)]
+)
+def test_group_by_first_last_big(
+    group_as_slice: bool, n: int, dtype: PolarsDataType
+) -> None:
+    group_by_first_last_test_impl(group_as_slice, n, dtype)
+
+
+def group_by_first_last_test_impl(
+    group_as_slice: bool, n: int, dtype: PolarsDataType
 ) -> None:
     idx = pl.Series([1, 2, 3, 4, 5], dtype=pl.Int32)
 
